library(tidyverse)
library(readxl)

#The next two packages are used for fitting power law, lognormal and 
#generalized gamma curves to the data, mainly for CoW warsizes but also for Mars.

library(poweRlaw)
library(gamlss)

#Load the Mars dataset from 
#  https://dataverse.harvard.edu/dataset.xhtml?persistentId=doi:10.7910/DVN/DUO7IE

library(readxl)
ProjectMarsV1_1 <- read_excel("ProjectMarsV1.1.xls")

#Load the CoW data

warsizes_CoW <- read.csv("Inter-StateWarData_v4.0.csv")

#The original HYDE data comes from here: https://dataportaal.pbl.nl/downloads/HYDE/
#We just took the population numbers and did linear interpolations for missing years
#We provide a file which contains both the original and the interpolated figures.

year_pop <- read.csv("HYDE with Interpolations from 1800.csv")

#Prepare the data so as to look just at war sizes - Sometimes an outcome of running
#this code is that the name of a war is rendered as a campaign within the war 
#rather than the proper name for the whole war. We tolerate this problem and only
#clean up the names when, e.g, we produce a table that requires correct names


warsizes <- ProjectMarsV1_1 %>% 
  group_by(warcode) %>% 
  summarise(kialow = sum(kialow), 
            kiahigh = sum(kiahigh),
            startyear = min(startyear),
            endyear = max(endyear),
            warname = min(warname))

#For some reason we wind up with a row 253 with NA's so we delete this row

warsizes <- warsizes[-253,]

#We now add population figures and create some synthetic variables
#year_pop gives a world population number for each year but there are years without
#wars starting and years with multiple wars starting so we need to do something
#to assign the population data to wars in a variable that we'll call "world_pop"
#Bear in mind that this is not a time series dataset but, rather, a list of wars


world_pop <- c()
for(i in 1:252) {
  world_pop <- c(world_pop,
                year_pop[warsizes[[i,4]]-1799,3])
}

#Add the population data to warsizes

warsizes <-   mutate(warsizes, world_pop)

#Create variables for kialow and kiahigh per 100,000 of world population

warsizes <- warsizes %>% 
  mutate(kialow_ph = 100000*kialow/world_pop,
         kiahigh_ph = 100000*kiahigh/world_pop)


#save the file - this is our main workhorse dataset for the paper 

write.csv(warsizes, "warsizes.csv")


#Now do the same things with the CoW data - this dataset will facilitate comparisons
#between the Mars and the CoW data

#Prepare the data so as to look just at war sizes 

warsizes_CoW <- warsizes_CoW %>% 
  group_by(WarNum) %>% 
  summarise(deaths = sum(BatDeath), 
            startyear = min(StartYear1),
            endyear = min(EndYear1),
            warname = min(WarName))


#year_pop give a world population number for each year but there are years without
#wars starting and years with multiple wars starting so we need to do something
#to assign the population data to wars in a variable that we'll call "world_pop_cow"
#Note this isn't the same as our world-pop variable above because the years with
#war starts are not identical for Mars and CoW

#We programmed this computation a little differently than we did for the Mars 
#data just to learn some new tricks but we could have substituted the above 
#code in here instead.

pop_assign <- function(x){
  temp <- year_pop%>%
    filter(year == warsizes_CoW%>%
             select(startyear)%>%
             slice(x)%>%
             pull()
    )
  temp[,"population.interpolated"] 
}

world_pop_CoW <- map_dbl(1:nrow(warsizes_CoW), pop_assign)

#Add the population data to warsizes

warsizes_CoW <-   mutate(warsizes_CoW, world_pop)

#Add a variable for deaths per 100,000 of world population

warsizes_CoW <- warsizes_CoW %>% 
  mutate(deaths_ph = 100000*deaths/world_pop)

#Save the data

write.csv(warsizes_CoW, "warsizes_cow.csv")


#The next bit of code creates a dataset that allows us to produce a standard log-log
#picture of the distribution of war sizes according to the low KIA estimates in the
#Mars data. For each KIA count (i.e, war size) the data gives the fraction of wars
#that exceed this KIA count. This is what's known as a "survival function". The 
#data set also gives survival functions for power law and lognormal fits to the data
#although we don't actually use these in the paper.

#Note - the word "all" is ubiqitous and, apparently, redundant in the rest of 
#the code in this file. It's there because in other code that we didn't ultimately
#integrate into this paper we analyze just the the post-1950 or post-WW2 data 
#so we don't use all the data.


#Create a discrete power law object for the KIALOW variable of the Mars data

all_low <- warsizes$kialow
all_low_pl   <- displ$new(all_low)

#We now fit a power law to the data. Note that the command "estimate_xmin" has
#a misleading name because the command doesn't only estimate x_min but, rather,
#estimates both x_min and the power law exponent. Also, the xmax setting can 
#also mislead. It's not setting the maximum value for the estimated power law
#as you might think. Rather, the routine sets an upper limit above which
#it treats the tail as a continuous power law distribution rather than as a 
#discrete power law. Basically, if this parameter is set too low than the search 
#doesn't perform well in the upper tail. But if you set it really high then the
#search takes forever. Finally, note that placing parentheses around everything
#prompts R to print the results. This is a computation intensive procedure and
#takes about 20 minutes to run on a decent laptop.

(est_all_low_pl <- estimate_xmin(all_low_pl, xmax = 100000000))

#The result is an xmin of 6,000 and an exponent of 1.67. 

#Since it takes 20 minutes to run this command here is all the output
#$gof [1] 0.03970737 $xmin [1] 6000 $pars [1] 1.672806 $ntail [1] 108 
#$distance [1] "ks [[7]] [1] "all_low_ln"

#Plug this xmin estimate into the discrete power law object. This is kind of a
#weird step but it is necessary so that the so-called "power law object" knows
#about our xmin estimate

all_low_pl$setXmin(est_all_low_pl)


#Extract the Mars data points and the discrete power law points.  The following 
#commands lead to survival functions, the first for the orginal (low estimate)
#data points and the second for the power law fit to these data points.

points_all_low <- plot(all_low_pl)

plfit_all_low <- lines(all_low_pl, col = 2)


#Now we add a lognormal fit now and store the results

all_low_ln <- dislnorm$new(all_low)
est_all_low_ln <- estimate_xmin(all_low_ln, xmax = 100000000)

#Here is the output. We include it here because it takes about 20 minutes
#to run the above commands
# $gof [1] 0.04460692 $xmin [1] 1400 $pars [1] 4.390374 3.415970
# $ntail [1] 199 $distance [1] "ks" attr(,"class") [1] "estimate_xmin"

#Again we need to plug the xmin estimate back in, this time to our lognormal
#object

all_low_ln$setXmin(est_all_low_ln)

#Save the survival function for the lognormal fit

lnfit_all_low <- lines(all_low_ln, col = 3)


#Export the data

fits_all_low <- bind_rows(points_all_low,plfit_all_low, lnfit_all_low)
fits_all_low  <- mutate(fits_all_low,c(rep("kialow",length(points_all_low$x)),
                                       rep("pl",length(plfit_all_low$x)),
                                       rep("ln", length(lnfit_all_low$x)))) 
fits_all_low <- fits_all_low %>%
  rename(cat = `c(...)`
  )

write.csv(fits_all_low,"fits_all_low.csv")

#Now we repeat all the above steps but for the high estimates of KIA in the 
#Mars data.


#Create a discrete power law object for the KIAHIGH variable of the Mars data

all_high <- warsizes$kiahigh
all_high_pl   <- displ$new(all_high)

(est_all_high_pl <- estimate_xmin(all_high_pl, xmax = 100000000))

#The result is an xmin of 6,000 and an exponent of 1.67. 

#Since it takes 20 minutes to run this command here is all the output
#$gof [1] 0.0402158 $xmin [1] 10575 $pars [1] 1.665702 $ntail [1] 100
#$distance [1] "ks" attr(,"class") [1] "estimate_xmin"

#Plug this xmin estimate into the discrete power law object. As noted above, 
#this is kind of a weird step but it is necessary so that the so-called 
#"power law object" knows about our xmin estimate

all_high_pl$setXmin(est_all_high_pl)


#Extract the Mars data points and the discrete power law points.  The following 
#commands lead to survival functions, the first for the orginal (high estimate)
#data points and the second for the power law fit to these data points.

points_all_high <- plot(all_high_pl)

plfit_all_high <- lines(all_high_pl, col = 2)


#Now we add a lognormal fit now and store the results

all_high_ln <- dislnorm$new(all_high)
est_all_high_ln <- estimate_xmin(all_high_ln, xmax = 100000000)

#Here is the output. We include it here because it takes about 20 minutes
#to run the above commands
#$gof [1] 0.0402158 $xmin [1] 10575 $pars [1] 1.665702 $ntail [1] 100
#$distance [1] "ks" attr(,"class") [1] "estimate_xmin"

#Again we need to plug the xmin estimate back into the lognormal object

all_high_ln$setXmin(est_all_high_ln)

#Save the survival function for the lognormal fit

lnfit_all_high <- lines(all_high_ln, col = 3)


#Export the data

fits_all_high <- bind_rows(points_all_high,plfit_all_high, lnfit_all_high)
fits_all_high  <- mutate(fits_all_high,c(rep("kiahigh",length(points_all_high$x)),
                                       rep("pl",length(plfit_all_high$x)),
                                       rep("ln", length(lnfit_all_high$x)))) 
fits_all_high <- fits_all_high %>%
  rename(cat = `c(...)`
  )

write.csv(fits_all_high,"fits_all_high.csv")

#Now we repeat the procedures again for the CoW data but we do a little bit more
#(See below)

all_CoW <- warsizes_CoW$deaths
all_CoW_pl   <- displ$new(all_CoW)

#We now fit a power law to the data. 

(est_all_CoW_pl <- estimate_xmin(all_CoW_pl, xmax = 100000000))


#Since it takes 20 minutes to run this command here is all the output

#$gof 0.06791739  $xmin [1] 6525 $pars [1] 1.513667 $ntail [1] 52 $distance
# [1] "ks" attr(,"class") [1] "estimate_xmin"

#Plug this xmin estimate into the discrete power law object. As noted above, 
#this is kind of a weird step but it is necessary so that the so-called 
#"power law object" knows about our xmin estimate

all_CoW_pl$setXmin(est_all_CoW_pl)


#Extract the Mars data points and the discrete power law points.  The following 
#commands lead to survival functions, the first for the orginal (CoW estimate)
#data points and the second for the power law fit to these data points.

points_all_CoW <- plot(all_CoW_pl)

plfit_all_CoW <- lines(all_CoW_pl, col = 2)


#Now we add a lognormal fit now and store the results

all_CoW_ln <- dislnorm$new(all_CoW)
est_all_CoW_ln <- estimate_xmin(all_CoW_ln, xmax = 100000000)

#Here is the output. We include it here because it takes about 20 minutes
#to run the above commands

# $gof [1] 0.06398308 $xmin [1] 1500 $pars [1] 4.722067 4.015130
# $ntail [1] 81 $distance [1] "ks" attr(,"class") [1] "estimate_xmin"

#Again we need to plug the xmin estimate back into the lognormal object

all_CoW_ln$setXmin(est_all_CoW_ln)

#Save the survival function for the lognormal fit

lnfit_all_CoW <- lines(all_CoW_ln, col = 3)

#Finally we use the gamlss package to derive points for the fit of a generalized
#gamma distribution to the CoW data. This distribution is the best fit among
#many fat-tailed distributions according to the AIC criterion

warvector <- sort(warsizes_CoW$deaths, decreasing = FALSE)

m1 <- fitDist(warvector)
m1$fits
m1$fails

temp <- function(f) {
  data.frame(
    x = f$y,
    p = do.call(
      paste0("p", f$family[1]), args = list(
        q = f$y,
        mu = f$mu,
        sigma = f$sigma,
        nu = f$nu
      )),
    pemp = gamlss::ECDF(f$y)(f$y)
  )
}

#Export the data

fits_all_CoW <- bind_rows(points_all_CoW,plfit_all_CoW, lnfit_all_CoW)
fits_all_CoW  <- mutate(fits_all_CoW,c(rep("kiaCoW",length(points_all_CoW$x)),
                                         rep("pl",length(plfit_all_CoW$x)),
                                         rep("ln", length(lnfit_all_CoW$x)))) 
fits_all_CoW <- fits_all_CoW %>%
  rename(cat = `c(...)`
  )


gg <- temp(m1) %>%
  mutate(survive_emp = 1-pemp, survive_gg = 1 - p)%>%
  subset(select =  -c(pemp, p))%>%
  distinct()
gg1 <- subset(gg, select = c(x, survive_emp))%>%rename(y = survive_emp)
gg2 <- subset(gg, select = c(x, survive_gg))%>%rename(y=survive_gg)



fits_all_CoW <- bind_rows(points_all_CoW,plfit_all_CoW, lnfit_all_CoW, gg1, gg2)
fits_all_CoW  <- mutate(fits_all_CoW,c(rep("kiaCoW",length(points_all_CoW$x)),
                                       rep("pl",length(plfit_all_CoW$x)),
                                       rep("ln", length(lnfit_all_CoW$x)),
                                       rep("empirical", length(gg1$x)),
                                       rep("gg",length(gg2$x))))
fits_all_CoW <- fits_all_CoW %>%
  rename(cat = `c(...)`
  )



write.csv(fits_all_CoW,"fits_all_CoW.csv")


#At the beginning of Appendix 2 we report results of simulations of data generated
#from power laws. These are computationally intensive so we ran them on GoogleCloud
#and we don't recommend running these unless you have very good computational power 

#This is the code we ran for the first set of power law simulations. There are 5 
#further sets of simulations but the code for each is almost exactly the same 
#(it's just about getting a larger sample) so we do not cut and past all 6 into
#this document

#For each alpha in c(1.5, 1.6, 1.7, 1.8, 1.9, 2.0) we generated 100 sets of 
#50 observations. 
#xmin was set at 1,000 
#discrete_max is set to 20,000,000.  This just means that the algorithm generates
#genuine discrete power law draws up to 20,000,000 and then resorts to treating
#the power law like a continuous one beyond that. In other words, the random numbers
#below 20 million are higher in quality than those above 20 million. This seems plenty
#high since the largest war in CoW has about 17 million battle deaths.
#I also set xmax equal to 20 million - this is actually the highest value that R 
#will consider for xmin so this one is set extravagantly high given that there is a
#true xmin of 1,000.


library(poweRlaw)

set.seed(199)

my_data <- c()

for (j in c(1.5, 1.6, 1.7, 1.8, 1.9, 2.0)) {
  for (i in 1:100) {
    my_data <- c(my_data,rpldis(50, 1000, j, discrete_max = 20000000))
  }
}

pl_data <- as.data.frame(matrix(my_data, nrow = 5000, ncol = 6))
colnames(pl_data) <- c("1.5", "1.6", "1.7", "1.8", "1.9", "2.0")


slopes <- c()
minima <- c()
for (k in 1:600) {
  pl_object <- displ$new(my_data[1:50+50*(k-1)])
  pl_fit <- estimate_xmin(pl_object, xmax = 20000000)
  slopes <- c(slopes, pl_fit$pars)
  minima <- c(minima, pl_fit$xmin)
}

alphas <- as.data.frame(matrix(slopes, nrow =100, ncol = 6))
colnames(alphas) <- c("1.5", "1.6", "1.7", "1.8", "1.9", "2.0")

x_mins <- as.data.frame(matrix(minima, nrow = 100, ncol = 6))
colnames(x_mins) <- c("1.5", "1.6", "1.7", "1.8", "1.9", "2.0")

write.csv(alphas, "alphas1.csv")
write.csv(x_mins, "x_mins1.csv")
write.csv(pl_data, "pl_data1.csv")



#Table 1A of the paper reports results of the simulations (above) for which we 
#generated sets of 50 draws from power laws, all with xmin set at 1,000, and 
#with exponents taking on the values 1.3, 1.4, ..., 2.1.  There are 200 draws of 
#50 for each exponent. We provide the code for these computations separately.  
#We note thatthese simulations require a lot of computation, taking about two 
#weeks on the google cloud. It's fully replicable but it's unlikely that anyone
#will be able to do it on a laptop.  We did it in chunks so that if something went 
#wrong we'd only lose a few days. The code below is just about merging the chunks into two
#composite files, one giving the raw data of all the simulations combined and the
#other giving the estimated exponents. 
#(Bear in mind that there is always a true exponent given by the chosen 
#parameter of the simulations and and estimated exponents. The whole point of 
#the excersise is to explore how they differ)


alphas1 <- read.csv("alphas1.csv")
alphas2 <- read.csv("alphas2.csv")
alphas3 <- read.csv("alphas3.csv")
alphas4 <- read.csv("alphas4.csv")
alphas5 <- read.csv("alphas5.csv")
alphas6 <- read.csv("alphas6.csv")

alphas_a <- rbind(alphas1, alphas2)
alphas_b <- rbind(alphas3, alphas4)
alphas_c <- rbind(alphas5, alphas6)

alphas_a <- alphas_a[-c(1)]
alphas_a  <- cbind(alphas_a, X = c(1:200))

alphas_b <- alphas_b[-c(1)]
alphas_b  <- cbind(alphas_b, X = c(1:200))

alphas_1000 <- merge(alphas_a, alphas_b, by = "X")

write.csv(alphas_1000,"alphas_1000.csv")


pl_data1 <- read.csv("pl_data1.csv")
pl_data2 <- read.csv("pl_data2.csv")
pl_data3 <- read.csv("pl_data3.csv")
pl_data4 <- read.csv("pl_data4.csv")
pl_data5 <- read.csv("pl_data5.csv")
pl_data6 <- read.csv("pl_data6.csv")

pl_data_a <- rbind(pl_data1, pl_data2)
pl_data_b <- rbind(pl_data3, pl_data4)
pl_data_c <- rbind(pl_data5, pl_data6)

pl_data_a <- pl_data_a[-c(1)]
pl_data_a  <- cbind(pl_data_a, X = c(1:200))

pl_data_b <- pl_data_b[-c(1)]
pl_data_b  <- cbind(pl_data_b, X = c(1:200))

pl_data_1000 <- merge(pl_data_a, pl_data_b, by = "X")

write.csv(pl_data_1000, "pl_data_1000.csv")


