# Note: reproduces code from app.
# Note: ulam takes quite a while longer than quap, due to compiling Stan code.
#       ~16 hours on my machine. Hence, I've saved the results as a .RData file.

# Libraries: ###################################################################
library(rethinking)
library(dplyr)
library(tidyr)
library(ggplot2)
library(scales)

engine <- if(require("cmdstanr")) {
  rethinking::ulam
} else {
  rethinking::quap
}

# Set-up: ######################################################################
# > runif(1) * 1E8
# [1] 27534605
set.seed(27534605)

# From Michael Spagat.
warsizes <- read.csv(file.path(".", "warsizes.csv"))

#TODO Can be fixed more appropriately later.
warsizes$startdate <- as.Date(paste0(warsizes$startyear, "-01-01"))

firstwar <- warsizes[which.min(warsizes$startdate),]
lastwar <- warsizes[which.max(warsizes$startdate),]

posterior <- as.Date("1950-01-01")

# Functions: ###################################################################

dataSubset <- function(
    data, thresh, start, stop, summarise = TRUE
) {
  temp <- data %>% dplyr::filter(
    Size >= thresh,
    startdate >= start,
    startdate < stop
  ) %>% dplyr::group_by(
    startyear
  ) %>% dplyr::summarise(
    WarCounts = dplyr::n(), .groups = "drop"
  ) %>% tidyr::complete(
    startyear = seq(as.numeric(strtrim(max(start, firstwar$startdate), 4)),
                    as.numeric(strtrim(min(stop, lastwar$startdate), 4)) - 1,
                    # Don't include the end (if it doesn't have any wars).
                    1),
    fill = list(WarCounts = 0)
  )
  if (summarise) {
    temp %>% dplyr::summarise(
      Number = sum(WarCounts),
      Years = length(WarCounts),
      Mean = Number / Years,
      InvYears = 1 / Years,
      StdDev = sqrt(Number / (Years^2))
    )
  } else {
    temp
  }
}

runAndRepeat <- function(model, data, limit = 10, quiet = FALSE) {
  success <- FALSE
  count <- 0
  starts <- 1 # Most rates are 1 plus or minus a bit
  while(!success && count <= limit) {
    if(count != 0) {starts <- runif(1, 0, 0.05)} # Most fail when data is rare.
    count <- count + 1
    tryCatch({
      output <- engine(
        model, data = data_post, start = list(Rate = starts),
        chains = 3, cores = 3
      )
      success <- TRUE
    }, error = function(e) {
      if(!quiet) {
        if(count == 1) {print(model); print(data)}
        print(paste(count, starts))
      }
    }
    )
  }
  if(!success) output <- engine(
    model, data = data_post
  )

  return(output)
}

# Main: ########################################################################
### Model Fitting: #############################################################
# Lazily expand our results. Better to preallocate, but then you would need to
# calculate ahead of time the number of records you would need, and R is pretty
# efficient about memory anyway.
if(exists("record")) rm(record)
if(exists("warsIncluded")) rm(warsIncluded)

ulamResults_loaded <- FALSE
if ((ulamResults <- "AlternativeFigures-ulamResults.RData") %in% dir()) {
  tryCatch({
    load(ulamResults)
    ulamResults_loaded <- TRUE
  })
}

if (!ulamResults_loaded) {
  for (RawPer in c("Raw", "Per 100,000")) {
    for (HighLow in c("Low", "High")) {
      data <- warsizes

      if (HighLow == "Low" && RawPer == "Raw") {
        data$Size <- data$kialow
      }
      if (HighLow == "High" && RawPer == "Raw") {
        data$Size <- data$kiahigh
      }
      if (HighLow == "Low" && RawPer == "Per 100,000") {
        data$Size <- data$kialow_ph
      }
      if (HighLow == "High" && RawPer == "Per 100,000") {
        data$Size <- data$kiahigh_ph
      }

      sortedSizes <- sort(unique(data$Size))

      for (thresh in sortedSizes) {
        # Priors
        data_1800 <- data %>% dataSubset(thresh, as.Date("1800-01-01"), posterior)
        data_1900 <- data %>% dataSubset(thresh, as.Date("1900-01-01"), posterior)
        data_post <- data %>% dataSubset(thresh, posterior, Sys.Date(),
                                         summarise = FALSE)

        # Stan model proxy
        model_unif <- alist(
          WarCounts ~ dpois(Rate),
          Rate ~ dunif(min = 0, max = 7)
        )

        if(isTRUE(all.equal(engine, rethinking::ulam))) {

          # dgamma does not play well with ulam.
          # dgamma2 does not play well with constants?
          # For others, good to supply a list with the

          model_1800 <- eval(substitute(alist(
            WarCounts ~ dpois(Rate),
            Rate ~ dgamma2(Mean, InvYears)
          ), data_1800))
          model_1900 <- eval(substitute(alist(
            WarCounts ~ dpois(Rate),
            Rate ~ dgamma2(Mean, InvYears)
          ), data_1900))

        } else if (isTRUE(all.equal(engine, rethinking::quap))) {

          model_1800 <- alist(
            WarCounts ~ dpois(Rate),
            Rate ~ dgamma(shape = data_1800$Number, rate = data_1800$Years)
          )
          model_1900 <- alist(
            WarCounts ~ dpois(Rate),
            Rate ~ dgamma(shape = data_1900$Number, rate = data_1900$Years)
          )

        }

        # If using the slim version of rethinking.
        # No posterior, and essentially no prior, so not much that can be done.
        # Replaced with the appropriate Stan calls with the full version.
        unifSkipFlag <-
          isTRUE(all.equal(engine, rethinking::quap)) &&
          sum(data_post$WarCounts) == 0
        if(!unifSkipFlag) {
          run_unif <- runAndRepeat(
            model_unif, data = data_post
          )
        }

        run_1800 <-
          tryCatch(
            runAndRepeat(
              model_1800, data = c(as.list(data_post), as.list(data_1800))
            )
            , error = function(e) {
              return(e)
            })

        if("error" %in% class(run_1800)) {
          # We're pushing quap in this scenario too much I believe.
          # (High variance quadratic approximations near 0 for non-zero param.)
          # Switching to proper Stan would fix, but then can't use slim rethink.
          next
        }

        run_1900 <- runAndRepeat(
          model_1900, data = c(as.list(data_post), as.list(data_1900))
        )

        # Posterior samples for checking probability of a decline.
        if(!unifSkipFlag) {
          samp_unif1800 <- rethinking::extract.samples(run_unif)
          samp_unif1900 <- rethinking::extract.samples(run_unif)
        }
        samp_1800 <- rethinking::extract.samples(run_1800)
        samp_1900 <- rethinking::extract.samples(run_1900)

        # Summary statistic with prior samples for comparison.
        # Note that we use the historical 1800 - 1950 for the uniform.
        record_temp <- data.frame(
          Threshold = thresh,
          Type = paste(HighLow, "Estimate,", RawPer),
          Prior = c(
            if(!unifSkipFlag) {
              c("Uniform vs. 1800", "Uniform vs. 1900")
            }
            ,
            "Gamma, 1800", "Gamma, 1900"),
          PriorParameter1 = c(
            if(!unifSkipFlag) {
              c(0, 0)
            }
            ,
            data_1800$Number, data_1900$Number),
          PriorParameter2 = c(
            if(!unifSkipFlag) {
              c(7, 7)
            }
            ,
            data_1800$Years, data_1900$Years),

          PriorYear = paste("Prior Start Year:", c(1800, 1900, 1800, 1900)),

          ProbabilityDecline = c(
            if(!unifSkipFlag) {
              c(
                sum(samp_unif1800$Rate < rgamma(
                  n = length(samp_unif1800$Rate),
                  shape = data_1800$Number,
                  rate = data_1800$Years)
                )/length(samp_unif1800$Rate),

                sum(samp_unif1900$Rate < rgamma(
                  n = length(samp_unif1900$Rate),
                  shape = data_1900$Number,
                  rate = data_1900$Years)
                )/length(samp_unif1900$Rate)
              )
            }
            ,

            sum(samp_1800$Rate < rgamma(
              n = length(samp_1800$Rate),
              shape = data_1800$Number,
              rate = data_1800$Years)
            )/length(samp_1800$Rate),

            sum(samp_1900$Rate < rgamma(
              n = length(samp_1900$Rate),
              shape = data_1900$Number,
              rate = data_1900$Years)
            )/length(samp_1900$Rate)
          )
        )

        if (!exists("record")) {
          record <- record_temp
        } else {
          record <- rbind(record, record_temp)
        }

        warsIncluded_temp <- data.frame(
          Threshold = thresh,
          Type = paste(HighLow, "Estimate,", RawPer),
          PercentageIncluded = sum(data$Size > thresh) / nrow(data)
        )
        if (!exists("warsIncluded")) {
          warsIncluded <- warsIncluded_temp
        } else {
          warsIncluded <- rbind(warsIncluded, warsIncluded_temp)
        }
      }
    }
  }
}

### Figures: ###################################################################
# Easiest to do in a ggplot format.
# Note that there can be a bit of debate as to whether the lines should be
# step-wise (akin to an empirical cdf) or (linearly) interpolated (below).

figure <- ggplot2::ggplot(
  record %>% dplyr::mutate(
    Type = factor(Type, levels = c(
      "Low Estimate, Raw", "High Estimate, Raw",
      "Low Estimate, Per 100,000", "High Estimate, Per 100,000"
    ))
  ),
  ggplot2::aes(
    x = Threshold,
    y = ProbabilityDecline,
    color = Prior, shape = Prior
  )
) + ggplot2::geom_point(
) + ggplot2::geom_line(
) + ggplot2::geom_line(
  data = warsIncluded %>% dplyr::mutate(
    Type = factor(Type, levels = c(
      "Low Estimate, Raw", "High Estimate, Raw",
      "Low Estimate, Per 100,000", "High Estimate, Per 100,000"
    ))
  ),
  mapping = ggplot2::aes(
    x = Threshold,
    y = PercentageIncluded,
    linetype = "Wars Included"
  ),
  inherit.aes = FALSE
  # linetype = "dashed"
) + ggplot2::scale_x_log10(
  name = "Threshold S"
) + ggplot2::scale_y_continuous(
  name = "P(Prior Rate > Posterior Rate)",
  sec.axis = ggplot2::sec_axis(
    ~., name = "Percentage Wars Included (Dashed)",
    labels = scales::label_percent()),
  limits = c(0, 1),
  labels = scales::label_percent()
) + ggplot2::facet_wrap(
  ~ Type + PriorYear, ncol = 2, scales = "free_x"
) + ggplot2::theme_bw(
) + ggplot2::scale_linetype_manual(
  values = "dashed", name = ""
)

ggplot2::ggsave(
  figure, filename = "Figure7_Alternative1.png",
  dpi = 320, width = 21, height = 28, units = "cm"
)

figure <- ggplot2::ggplot(
  record %>% dplyr::mutate(
    Type = factor(Type, levels = c(
      "Low Estimate, Raw", "High Estimate, Raw",
      "Low Estimate, Per 100,000", "High Estimate, Per 100,000"
    ))
  ),
  ggplot2::aes(
    x = Threshold,
    y = ProbabilityDecline,
    color = Prior, shape = Prior,
    linetype = "P(Prior Rate > Rate)"
  )
) + ggplot2::geom_point(
) + ggplot2::geom_line(
) + ggplot2::geom_line(
  data = warsIncluded %>% dplyr::mutate(
    Type = factor(Type, levels = c(
      "Low Estimate, Raw", "High Estimate, Raw",
      "Low Estimate, Per 100,000", "High Estimate, Per 100,000"
    ))
  ),
  mapping = ggplot2::aes(
    x = Threshold,
    y = PercentageIncluded,
    linetype = "Wars Included"
  ),
  inherit.aes = FALSE
  # linetype = "dashed"
) + ggplot2::scale_x_log10(
  name = "Threshold S"
) + ggplot2::scale_y_continuous(
  name = "Probability of Rate Shift, Percentage Wars Included",
  # sec.axis = ggplot2::sec_axis(
  #   ~., name = "Percentage Wars Included (Dashed)",
  #   labels = scales::label_percent()),
  limits = c(0, 1),
  labels = scales::label_percent()
) + ggplot2::facet_wrap(
  ~ Type + PriorYear, ncol = 2, scales = "free_x"
) + ggplot2::theme_bw(
) + ggplot2::scale_linetype(
  name = ""
)

ggplot2::ggsave(
  figure, filename = "Figure7_Alternative2.png",
  dpi = 320, width = 21, height = 28, units = "cm"
)
