options(scipen=999)
library(tidyverse)
library(gt)   #For tables
library(webshot) #For saving images of gt tables
library(tidyquant)  #For getting a moving average in figure 2
library(ggpubr)  #For stacking figures

#Load data 
#Note - We provide all the datasets to be loaded here.  In addition, there is 
#a separate "Data Prep" script that shows how there were created from the original
#Mars and CoW data

#The "warsizes" dataset is derived from the Mars dataset.  It collapses the data
#into a list of the 252 wars with a few variables on each war, in particular,
#low and high estimates of killed in action in both raw and population-adjusted
#terms plus start years for each war.

warsizes <- read.csv("warsizes.csv")

#Next is the same as above but for the Correlates of War dataset.  Note that CoW
#has just one estimate for total battle deaths for each war, not a high and a low one.

cow <- read.csv("warsizes_cow.csv")

#The next dataset is provided on the Mars data website.  It's a list of wars that
#are included in Mars but not in CoW together with reasons for exclusion, usually
#because they are not conventional wars.

missing_from_CoW <- read.csv("Missing from Cow.csv")  

#The next three datasets were produced by us using several packages and the CoW
#and Mars data. We provide the code separately
#It takes a while to run the code so we make the final product available here. 
#They give fits using power law, expontential and generalized gamma distributions
#to, respectively, the CoW, Mars (low estimates) and Mars (high estimates) data

fits_all_CoW <- read.csv("fits_all_CoW.csv")

fits_all_low <- read.csv("fits_all_low.csv")

fits_all_high <- read.csv("fits_all_high.csv")


#The next two tables are giving war arrival rates for Mars and CoW for a few
#periods
#create a dataframe with annual totals of killings and killings per 100,000 

annual_totals <- warsizes %>% 
  group_by(startyear) %>% 
  summarise(kialow = sum(kialow), 
            kiahigh = sum(kiahigh),
            kialow_ph = sum(kialow_ph),
            kiahigh_ph = sum(kiahigh_ph))

for (i in 1800:2011)
{
  if (i %in% annual_totals$startyear){
    annual_totals <- annual_totals
  }
  else {
    annual_totals <- rbind(annual_totals, c(i,0,0,0,0,0))
  }
}

#make sure the years are in the right order
annual_totals <- annual_totals[order(annual_totals$startyear),]

#create a function to calculate per annum figures for any time period and any measure

means_by_period <- function(beginning, end, measure)  {
  ifelse(measure == "kialow",
         sum(select(annual_totals%>%filter(startyear>=beginning & startyear <= end),
                    kialow))/(end-beginning+1),
         ifelse(measure == "kiahigh",
                sum(select(annual_totals%>%filter(startyear>=beginning & startyear <= end),
                           kiahigh))/(end-beginning+1),
                ifelse(measure == "kialow_ph",
                       sum(select(annual_totals%>%filter(startyear>=beginning & startyear <= end),
                                  kialow_ph))/(end-beginning+1),
                       sum(select(annual_totals%>%filter(startyear>=beginning & startyear <= end),
                                  kiahigh_ph))/(end-beginning+1))))
}

#Build a table that will give the per annum figures for every measure and for
#three time periods

Periods <- c("1800-1899", "1900-1950", "1951-2011")

Low <- c(means_by_period(1800, 1899, "kialow"), 
         means_by_period(1900, 1950,"kialow"),
         means_by_period(1951, 2011, "kialow"))
High <- c(means_by_period(1800, 1899, "kiahigh"), 
          means_by_period(1900, 1950,"kiahigh"),
          means_by_period(1951, 2011, "kiahigh"))
low_ph <- c(means_by_period(1800, 1899, "kialow_ph"), 
            means_by_period(1900, 1950,"kialow_ph"),
            means_by_period(1951, 2011, "kialow_ph"))
high_ph <- c(means_by_period(1800, 1899, "kiahigh_ph"), 
             means_by_period(1900, 1950,"kiahigh_ph"),
             means_by_period(1951, 2011, "kiahigh_ph"))

#Build a function that will allow us to calculate annual arrival rates for wars of
#all sizes, for all measures and for any time period.  Note - this provides more
#power than we need for building table one but we'll surely make use of all the 
#features at some point


war_rate <- function(beginning, end, bottom, measure) {
  ifelse(measure == "kialow",
         warsizes%>%filter(startyear>=beginning & startyear <= end & kialow >bottom)%>%
           nrow()/(end-beginning+1),
         ifelse(measure == "kiahigh",
                warsizes%>%filter(startyear>=beginning & startyear <= end & kiahigh >bottom)%>%
                  nrow()/(end-beginning+1),
                ifelse(measure == "kialow_ph",
                       warsizes%>%filter(startyear>=beginning & startyear <= end & kialow_ph >bottom)%>%
                         nrow()/(end-beginning+1),
                       warsizes%>%filter(startyear>=beginning & startyear <= end & kiahigh_ph >bottom)%>%
                         nrow()/(end-beginning+1))))
}


#Note - since we're looking at the arrival rate for all wars in each time period,
#i.e., wars with KIA counts greater than 0 it doesn't matter which measure we use
#so we just do it for kialow

arrivals <- c(war_rate(1800, 1899, 0, "kialow"), war_rate(1900, 1950, 0, "kialow"),
              war_rate(1951, 2011, 0, "kialow"))

#put together the data frame

kiapa <- mutate(mutate(mutate(mutate(mutate(as.data.frame(Periods), Low), High), 
                              low_ph), high_ph), arrivals)
#Here's the table

table1 <- kiapa %>%
  gt() %>%
  tab_header(
    title = "Table 1 - Killed in Action and Number of Wars Per Annum for Three Time Periods",
  )%>%
  cols_label(low_ph = "Low per 100,000")%>%
  cols_label(high_ph = "High per 100,000")%>%
  cols_label(arrivals = "Number of Wars")%>%
  tab_source_note(
    source_note = "Data: Project Mars")%>%
  tab_footnote(
    footnote = "Using the low estimates of killed in action",
    locations = cells_column_labels(columns = Low
    )
  )%>%
  tab_footnote(
    footnote = "Using the high estimates of killed in action",
    locations = cells_column_labels(columns = High
    )
  )%>%
  tab_footnote(
    footnote = "Using the low estimates of killed in action per 100,000 
    of world population",
    locations = cells_column_labels(columns = low_ph
    )
  )%>%
  tab_footnote(
    footnote = "Using the high estimates of killed in action per 100,000 
    of world population",
    locations = cells_column_labels(columns = high_ph
    )
  )%>%
  fmt_number(c(Low, High),decimals = 0, sep_mark = "," )%>%
  fmt_number(c(arrivals, low_ph, high_ph),decimals = 1, sep_mark = "," )

gtsave(table1, "table1.png")

#Now do the same table but with correlates of war


#Get annual totals for CoW
annual_totals_cow <- cow %>% 
  group_by(startyear) %>% 
  summarise(deaths = sum(deaths),
            deaths_ph = sum(deaths_ph))

#fill in 0's for years when no war started
for (i in 1816:2007)
{
  if (i %in% annual_totals_cow$startyear){
    annual_totals_cow <- annual_totals_cow
  }
  else {
    annual_totals_cow <- rbind(annual_totals_cow, c(i,0,0))
  }
}


#make sure the years are in the right order
annual_totals_cow <- annual_totals_cow[order(annual_totals_cow$startyear),]


means_by_period_cow <- function(beginning, end, measure)  {
  ifelse(measure == "deaths",
         sum(select(annual_totals_cow%>%filter(startyear>=beginning & startyear <= end),
                    deaths))/(end-beginning+1),
         sum(select(annual_totals_cow%>%filter(startyear>=beginning & startyear <= end),
                    deaths_ph))/(end-beginning+1))
}

Periods <- c("1800-1899", "1900-1950", "1951-2011")

Battle_Deaths <- c(means_by_period_cow(1800, 1899, "deaths"), 
                   means_by_period_cow(1900, 1950,"deaths"),
                   means_by_period_cow(1951, 2011, "deaths"))
Battle_Deaths_ph <- c(means_by_period_cow(1800, 1899, "deaths_ph"), 
                      means_by_period_cow(1900, 1950,"deaths_ph"),
                      means_by_period_cow(1951, 2011, "deaths_ph"))


war_rate_cow <- function(beginning, end, bottom, measure) {
  ifelse(measure == "deaths",
         cow%>%filter(startyear>=beginning & startyear <= end & deaths >bottom)%>%
           nrow()/(end-beginning+1),
         cow%>%filter(startyear>=beginning & startyear <= end & deaths_ph >bottom)%>%
           nrow()/(end-beginning+1))
}

arrivals_cow <- c(war_rate_cow(1800, 1899, 0, "deaths"), 
                  war_rate_cow(1900, 1950, 0, "deaths"),
                  war_rate_cow(1951, 2011, 0, "deaths"))

kiapa_cow <- (mutate(mutate(mutate(as.data.frame(Periods), Battle_Deaths), 
                            Battle_Deaths_ph), arrivals_cow))


table2 <- kiapa_cow %>%
  gt() %>%
  tab_header(
    title = "Table 2 - Battle Deaths and Number of Wars Per Annum for Various Time Periods",
  )%>%
  cols_label(Battle_Deaths = "Battle Deaths")%>%
  cols_label(Battle_Deaths_ph = "Battle Deaths per 100,000")%>%
  cols_label(arrivals_cow = "Number of Wars")%>%
  tab_source_note(
    source_note = "Data: Correlates of War")%>%
  fmt_number(Battle_Deaths,decimals = 0, sep_mark = "," )%>%
  fmt_number(c(Battle_Deaths_ph,arrivals_cow),decimals = 1, sep_mark = "," )

gtsave(table2, "table2.png")


#Now let's get a handle on what's in Mars by missing in CoW by making a table
#of the biggest 20 such cases

table3 <- missing_from_CoW %>%
  gt() %>%
  tab_header(
    title = "Table 3 - The Twenty Biggest Wars Excluded from CoW",
    subtitle = "Ordered by the low estimates of Killed in Action"
  )%>%
  cols_label(War.Name = "War Name")%>%
  cols_label(Start.Year = "Start Year")%>%
  cols_label(End.Year = "End Year")%>%
  tab_source_note(
    source_note = "Data: Project Mars and CoW")%>%
  fmt_number(c(kialow, kiahigh),decimals = 0, sep_mark = "," )

gtsave(table3, "table3.png")


#Let's look at summary statistics by period now

period_after <- warsizes%>% filter(startyear>1965 & startyear <2012)

period_before <- warsizes%>% filter(startyear>1799 & startyear <1951)

kia_variables <- list(round(warsizes$kialow, 0),period_before$kialow,
                      period_after$kialow,warsizes$kiahigh,period_before$kiahigh,
                      period_after$kiahigh,round(warsizes$kialow_ph,3),
                      period_before$kialow_ph,period_after$kialow_ph,
                      warsizes$kiahigh_ph,period_before$kiahigh_ph,
                      period_after$kiahigh_ph)

dist_table <- c()
for(i in 1:12){
  dist_table <- c(dist_table, fivenum(kia_variables[[i]]), 
                  mean(kia_variables[[i]]))
}

length(kia_variables[[1]])

dist_table <- as.data.frame(t(matrix(dist_table, nrow = 6)))
colnames(dist_table) <-c("Minimum", "first_quartile", "median", "third quartile", 
                         "Maximum", "Mean")
measurements <- c("1800-2011","1800-1950","1951-2011",
                  "1800-2011","1800-1950","1951-2011",
                  "1800-2011","1800-1950","1951-2011",
                  "1800-2011","1800-1950","1951-2011")
dist_table <- cbind(measurements, dist_table)


kia_variables <- dist_table %>%
  gt() %>%
  fmt_number(columns = 2:7,rows = 1:6, decimals = 0, sep_mark = ",")%>%
  fmt_number(columns = 2:7,rows = 7:12, decimals = 3)%>%
  cols_label(measurements= "", first_quartile = "Quartile 1", median = "Median", 
             `third quartile` = "Quartile 3")%>%
  tab_row_group(label = "High Estimate of Killed in Action per 100,000",rows = 10:12)%>%
  tab_row_group(label = "Low Estimate of Killed in Action per 100,000",rows = 7:9)%>%
  tab_row_group(label = "High Estimate of Killed in Action",rows = 4:6)%>%
  tab_row_group(label = "Low Estimate of Killed in Action",rows = 1:3)%>%
  tab_style(style = cell_fill(),
            locations = cells_body(columns = 3:5, rows= c(2:3,5:6,8:9, 11:12)))%>%
  tab_source_note(
    source_note = "Data: Project Mars. There are 203 wars before and 49 wars after 1950.")%>%
  tab_header(
    title = "Table 4 - Summary Statistics for the Distributions of KIA Counts",
    subtitle = "Various KIA Measures and Time Periods")

gtsave(kia_variables, "table4.png")

#Now we turn to a robustness exercise - how many wars do there have to be between
#2012 and 2030 in order to push various p values from below 0.05 to above 0.05

x_median_extended <- war_rate(1800, 1950, 4177, "kialow")*81

i <- 0
while (ppois(i,x_median_extended) <= 0.05) {
  i <- i+1
}
wars_required <- i - war_rate(1951, 2011, 4177, "kialow")*61

wars_needed_low <- function(size){
  i <- 0
  while (ppois(i,war_rate(1800, 1950, size, "kialow")*81) <= 0.05) {
    i <- i+1
  }
  i - war_rate(1951, 2011, size, "kialow")*61
}

wars_needed_high <- function(size){
  i <- 0
  while (ppois(i,war_rate(1800, 1950, size, "kiahigh")*81) <= 0.05) {
    i <- i+1
  }
  i - war_rate(1951, 2011, size, "kiahigh")*61
}
wars_needed_low_ph <- function(size){
  i <- 0
  while (ppois(i,war_rate(1800, 1950, size, "kialow_ph")*81) <= 0.05) {
    i <- i+1
  }
  i - war_rate(1951, 2011, size, "kialow_ph")*61
}

wars_needed_high_ph <- function(size){
  i <- 0
  while (ppois(i,war_rate(1800, 1950, size, "kiahigh_ph")*81) <= 0.05) {
    i <- i+1
  }
  i - war_rate(1951, 2011, size, "kiahigh_ph")*61
}

wars_needed <- function(size, measure){
  ifelse(measure == "kialow", wars_needed_low(size),
         ifelse(measure == "kiahigh", wars_needed_high(size),
                ifelse(measure == "kialow_ph", wars_needed_low_ph(size),
                       wars_needed_high_ph(size))))
}


table_variables <- list(warsizes$kialow, warsizes$kiahigh, 
                        warsizes$kialow_ph, warsizes$kiahigh_ph)

robust_table <- c()
for(i in 1:4){
  robust_table <- c(robust_table, fivenum(table_variables[[i]]))
}
robust_table <- as.data.frame(matrix(robust_table, nrow = 5))%>%
  filter(! row_number() %in% c(1,5))

robust_table <- robust_table %>% mutate(kialow = wars_needed(V1,"kialow"))


temp <- 
  c(wars_needed(robust_table[1,1], "kialow"), wars_needed(robust_table[2,1], "kialow"),
    wars_needed(robust_table[3,1], "kialow"),
    wars_needed(robust_table[1,2], "kiahigh"), wars_needed(robust_table[2,2], "kiahigh"),
    wars_needed(robust_table[3,2], "kiahigh"),
    wars_needed(robust_table[1,3], "kialow_ph"), wars_needed(robust_table[2,3], "kialow_ph"),
    wars_needed(robust_table[3,3], "kialow_ph"),
    wars_needed(robust_table[1,4], "kiahigh_ph"), wars_needed(robust_table[2,4], "kiahigh_ph"),
    wars_needed(robust_table[3,4], "kiahigh_ph")
  )

robust_counts <- as.data.frame(t(matrix(temp, nrow = 3)))%>%
  mutate(measures = c("Low Estimates of KIA", "High Estimates of KIA", 
                      "Low Estimates of KIA per 100,000", 
                      "High Estimates of KIA per 100,000"))%>%
  relocate(measures)
colnames(robust_counts) <- c("Measures", "Above Quartile 1", "Above the Median",
                             "Above Quartile 3")

table5 <- robust_counts %>%
  gt() %>%
  tab_spanner(
    label = "Ranges",
    columns = c("Above Quartile 1", "Above the Median",
                "Above Quartile 3"))%>%
  tab_source_note(
    source_note = "Each table entry gives the number of wars required to push the p value on the corresponding no-change hypothesis above 0.05. For example, if there are 34 wars between 2012 and 2031 with low estimates of KIA counts above the 1st quartile of KIA counts for 1800-2011 then we would (just) fail to reject the no-change hypothesis for wars in this size range.  
      Data: Project Mars")%>%
  tab_header(
    title = "Table 5 - Numbers of Wars between 2012 and 2031 Required for Non-Rejection of No-Change Hypotheses",
    subtitle = "Various KIA Measures and Time Periods")

gtsave(table5, "table5.png")

#Now let's move on to the graphs.  


#Here is graph 1 for the main paper. It is similar to the graph 2Appendix for 
#the appendix but this one displays a data point for each war.


#The raw time series
raw_plot <- warsizes %>%
  ggplot(aes(x = startyear, y = kialow)) +
  geom_point() +
  labs(title = "The Start Year Versus the KIA Count for Each War", 
       y = "KIA (low)", x = "Year") + 
  theme(panel.grid.major = element_blank(), 
        panel.grid.minor = element_blank(),
        panel.background = element_blank(), 
        axis.line = element_line(colour = "black"),
        plot.margin = unit(c(1,0.2,0.2,0.2), "cm"))



#This is visually the same as the old one

#The world wars dominate to the point where the picture isn't very interesting

#logging the y axis doesn't seem to produce much insight
log_plot <- warsizes %>%
  ggplot(aes(x = startyear, y = kialow)) +
  geom_point() +
  scale_y_log10() +
  labs(title = "The Start Year Versus the KIA Count for Each War",
  subtitle = "Logged Y Axis", 
       y = "KIA (low)", x = "Year") + 
  theme(panel.grid.major = element_blank(), 
        panel.grid.minor = element_blank(),
        panel.background = element_blank(), 
        axis.line = element_line(colour = "black"))

#again there's no visual change from the earlier picture



distribution <- fits_all_low%>%
  ggplot(aes(x = x, y = y))+
  geom_point(data = filter(fits_all_low, cat == "kialow"))+
  scale_y_continuous(trans='log10') +
  scale_x_continuous(trans='log10') +
  labs(title = "The Distribution of KIA Counts on log-log Axes",
       y = "Prob(X > x)", 
       x = "Battle Death Counts")+
  theme(panel.grid.major = element_blank(), 
        panel.grid.minor = element_blank(),
        panel.background = element_blank(), 
        axis.line = element_line(colour = "black"),
        plot.margin = unit(c(0.1,0.1,0.1,1.00), "cm")
        )

#Now put it together in a single figure

figure1 <- ggarrange(raw_plot, log_plot, distribution, 
                     vjust = 4, font.label = list(size = 8),
                     labels = c("a", 
                                "b",
                                "c"
                     ),
                     ncol = 1, nrow = 3)
figure1 <- annotate_figure(figure1,
                           fig.lab = "Figure 1 - Three Views of the Mars KIA Counts for all Wars",
                           fig.lab.size = 14
)

ggsave("graph1.png", device = "png" ,dpi = 700, width = 7.5, height = 10, units = "in")



#Now let's try a picture of means from each year up to 2011

#first we need to create variables for forward sums

forward_sums_kialow <- c(sum(annual_totals$kialow),sum(annual_totals$kialow)-cumsum(annual_totals$kialow))
forward_sums_kialow <- forward_sums_kialow[1:212]

forward_sums_kialow_annualized <- c()
for (i in 1:212) {
  forward_sums_kialow_annualized <- 
    c(forward_sums_kialow_annualized,forward_sums_kialow[i]/(213 - i))  
}

annual_totals <- mutate(annual_totals, forward_sums_kialow_annualized)

forwardA <- ggplot(data = annual_totals)+
  labs(y = "", x = "Year") + 
  geom_line(mapping = aes(x = startyear, 
                          y = forward_sums_kialow_annualized)) +
  geom_line(mapping = aes(x = startyear, 
                          y = forward_sums_kialow_annualized[1]), 
            linetype = "dashed") +
  geom_line(mapping = aes(x = startyear, 
                          y = means_by_period(1800, 1899, "kialow")),
            linetype = "dashed") +
  geom_line(mapping = aes(x = startyear, 
                          y = means_by_period(1951, 2011, "kialow")),
            linetype = "dashed") +
  geom_text(x = 1985, y = 178000, label = "mean for the whole period", cex = 2) +
  geom_text(x = 1835.5, y = 20000, label = "mean for the 19th century", cex = 2) +
  geom_text(x = 1830, y = 35000, label = "mean after 1950", cex = 2) +
  geom_text(x = 1972, y = 42000, label = "Vietnam War", cex = 2) +
  geom_text(x = 1957, y = 43000, label = "Korean War", cex = 2) +
  geom_text(x = 1956, y = 59000, label = "Chinese Civil War", cex = 2) +
  geom_text(x = 1917, y = 357000, label = "WW1", cex = 2) +
  geom_text(x = 1939, y = 358000, label = "WW2", cex = 2) +
  theme(panel.grid.major = element_blank(), panel.grid.minor = element_blank(),
        panel.background = element_blank(), axis.line = element_line(colour = "black"),
        plot.margin = unit(c(1.0,0.1,0.1,0.1), "cm"))

#Lets try it with high estimates now


forward_sums_kiahigh <- c(sum(annual_totals$kiahigh),sum(annual_totals$kiahigh)-cumsum(annual_totals$kiahigh))
forward_sums_kiahigh <- forward_sums_kiahigh[1:212]

forward_sums_kiahigh_annualized <- c()
for (i in 1:212) {
  forward_sums_kiahigh_annualized <- 
    c(forward_sums_kiahigh_annualized,forward_sums_kiahigh[i]/(213 - i))  
}

annual_totals <- mutate(annual_totals, forward_sums_kiahigh_annualized)


forwardB <- ggplot(data = annual_totals)+
  geom_line(mapping = aes(x = startyear, 
                          y = forward_sums_kiahigh_annualized)) +
  geom_line(mapping = aes(x = startyear, 
                          y = forward_sums_kiahigh_annualized[1]), 
            linetype = "dashed")  +
  geom_line(mapping = aes(x = startyear, 
                          y = means_by_period(1800, 1899, "kiahigh")),
            linetype = "dashed") +
  geom_line(mapping = aes(x = startyear, 
                          y = means_by_period(1951, 2011, "kialow")),
            linetype = "dashed") +
  geom_text(x = 1985, y = 263000, label = "mean for the whole period", cex = 2)+
  geom_text(x = 1835.4, y = 30000, label = "mean for the 19th century", cex = 2) +
  geom_text(x = 1830, y = 57000, label = "mean after 1950", cex = 2) +
  geom_text(x = 1917, y = 528000, label = "WW1", cex = 2) +
  geom_text(x = 1939, y = 540000, label = "WW2", cex = 2) +
  geom_text(x = 1957, y = 67500, label = "Korean War", cex = 2) +
  geom_text(x = 1972, y = 64000, label = "Vietnam War", cex = 2) +
  geom_text(x = 1956, y = 94000, label = "Chinese Civil War", cex = 2) +
  labs(y = "", x = "Year") + 
  theme(panel.grid.major = element_blank(), panel.grid.minor = element_blank(),
        panel.background = element_blank(), axis.line = element_line(colour = "black"),
        plot.margin = unit(c(1.0,0.1,0.1,0.1), "cm"))

figure2 <- ggarrange(forwardA,forwardB, 
                    vjust = 5, font.label = list(size = 8),
                    labels = c("KIALOW", 
                               "KIAHIGH"),
                    ncol = 1, nrow = 2)
annotate_figure(figure2,
                fig.lab = "Figure 2 - Annualized Averages for Killed in Action from each Year Forward to 2011", 
                fig.lab.face = "bold"
)

ggsave("graph2.png", device = "png" ,dpi = 700, width = 7.5, height = 10, units = "in")

#Note - Do the same picture for the appendix with figures per 100,000

#Next is a picture comparing actual numbers of wars in every size range with
#predicted numbers of wars complete with lower confidence limits.


#Create a vector giving the number of wars exceeding the size of each post-1950 war
number_over_low <- c()
for(i in 1:252) {
  if(warsizes[i,"startyear"] > 1950) {
    temporary <- warsizes[i, "kialow"]
    number_over_low <- c(number_over_low, 
                         nrow(filter(warsizes, startyear < 1951, kialow > temporary)) )
  }
}


#The idea here is that for each post-1950 war you calculate the number of 
#post-1950 wars there should be at or above that size if post-1950 wars in 
#that size range arrive at the rate they did before 1950

number_low_post <- c(1:length(number_over_low))
number_over_low_sorted <- sort(number_over_low)
pred_low_central <- (number_over_low_sorted)*61/151
pred_low_upper  <- qpois(.975, pred_low_central, lower.tail = TRUE)
pred_low_lower  <- qpois(.025, pred_low_central, lower.tail = TRUE)
pred_low_lower_.05 <- qpois(.05, pred_low_central, lower.tail = TRUE)
pred_low_lower_.10 <- qpois(.10, pred_low_central, lower.tail = TRUE)


kia_low_post <- warsizes %>% filter(startyear >1950)%>%select(kialow)
temp  <- arrange(kia_low_post, kialow)
kia_low_post_asc <- temp$kialow
number_low_post_desc <- sort(number_low_post, decreasing = TRUE)
pred_low_central_desc <- sort(pred_low_central, decreasing = TRUE)
pred_low_upper_desc <- sort(pred_low_upper, decreasing = TRUE)
pred_low_lower_desc <- sort(pred_low_lower, decreasing = TRUE)
pred_low_lower_.05_desc <- sort(pred_low_lower_.05, decreasing = TRUE)
pred_low_lower_.10_desc <- sort(pred_low_lower_.10, decreasing = TRUE)


cardinal_df_low <- data.frame(kia_low_post_asc,number_low_post_desc,
                              pred_low_central_desc, pred_low_lower_desc,
                              pred_low_upper_desc)


predictionsA <- ggplot(cardinal_df_low, aes(x= kia_low_post_asc ))+
  geom_line(aes(y = pred_low_central_desc),linetype = "dotted") +
  geom_line(aes(y = number_low_post_desc)) +
  geom_line(aes(y = pred_low_lower_desc), linetype="dashed") +
  geom_line(aes(y = pred_low_lower_.10_desc), linetype="dashed") +
  scale_x_continuous(trans='log10', breaks = c(500, 1000, 5000, 10000, 50000,
                                               100000, 500000, 1000000)) +
  labs(x = "Mars Low Estimates of Killed in Action",
       y = "Predicted and Actual Counts of Post-1950 Wars") + 
  theme(plot.title = element_text(size=12)) +
  theme(plot.subtitle = element_text(size = 10)) +
  annotate(geom = "text", x = 3200, y = 63, label = "predicted number of wars",
           size = 2)+
  annotate(geom = "text", x = 610, y = 53, label = "lower limit (0.025)",
           size = 2)+
  annotate(geom = "text", x = 550, y = 67, label = "lower limit (0.1)",
           size = 2)+
  annotate(geom = "text", x = 1000, y = 33, label = "actual number of wars",
           size = 2)+
  theme(panel.background = element_blank(), 
        panel.grid.major = element_line(colour = "grey"),
        panel.grid.minor = element_blank(),,
        axis.line = element_line(colour = "black"),
        plot.margin = unit(c(1.0,0.2,0.2,0.2), "cm"))


#Now let's do the same thing but for the high estimates

number_over_high <- c()
for(i in 1:252) {
  if(warsizes[i,"startyear"] > 1950) {
    temporary <- warsizes[i, "kiahigh"]
    number_over_high <- c(number_over_high, 
                          nrow(filter(warsizes, startyear < 1951, kiahigh > temporary)) )
  }
}



number_high_post <- c(1:length(number_over_high))
number_over_high_sorted <- sort(number_over_high)
pred_high <- (number_over_high_sorted)*61/151


number_high_post <- c(1:length(number_over_high))
number_over_high_sorted <- sort(number_over_high)
pred_high_central <- (number_over_high_sorted)*61/151
pred_high_upper  <- qpois(.975, pred_high_central, lower.tail = TRUE)
pred_high_lower  <- qpois(.025, pred_high_central, lower.tail = TRUE)
pred_high_lower_.05 <- qpois(.05, pred_high_central, lower.tail = TRUE)
pred_high_lower_.10 <- qpois(.10, pred_high_central, lower.tail = TRUE)



kia_high_post <- warsizes %>% filter(startyear >1950)%>%select(kiahigh)
temp  <- arrange(kia_high_post, kiahigh)
kia_high_post_asc <- temp$kiahigh
number_high_post_desc <- sort(number_high_post, decreasing = TRUE)
pred_high_central_desc <- sort(pred_high_central, decreasing = TRUE)
pred_high_upper_desc <- sort(pred_high_upper, decreasing = TRUE)
pred_high_lower_desc <- sort(pred_high_lower, decreasing = TRUE)
pred_high_lower_.05_desc <- sort(pred_high_lower_.05, decreasing = TRUE)
pred_high_lower_.10_desc <- sort(pred_high_lower_.10, decreasing = TRUE)


cardinal_df_high <- data.frame(kia_high_post_asc,number_high_post_desc,
                               pred_high_central_desc, pred_high_lower_desc,
                               pred_high_upper_desc)



predictionsB <-ggplot(cardinal_df_high, aes(x= kia_high_post_asc ))+
  geom_line(aes(y = pred_high_central_desc),linetype = "dotted") +
  geom_line(aes(y = number_high_post_desc)) +
  geom_line(aes(y = pred_high_lower_desc), linetype="dashed") +
  geom_line(aes(y = pred_high_lower_.10_desc), linetype="dashed") +
  scale_x_continuous(trans='log10',breaks = c(500, 1000, 5000, 10000, 50000,
                                             100000, 500000, 1000000)) +
  labs(x = "Mars High Estimates of Killed in Action",
       y = "Predicted and Actual Counts of Post-1950 Wars") + 
  theme(plot.title = element_text(size=12)) +
  theme(plot.subtitle = element_text(size = 10)) +
  annotate(geom = "text", x = 5500, y = 62, label = "predicted number of wars",
           size = 2)+
  annotate(geom = "text", x = 1200, y = 68, label = "lower limit (0.1)",
           size = 2)+
  annotate(geom = "text", x = 500, y = 60, label = "lower limit (0.025)",
           size = 2)+
  annotate(geom = "text", x = 2000, y = 33, label = "actual number of wars",
           size = 2)+
  theme(panel.grid.major = element_line(colour = "grey"), 
        panel.grid.minor = element_blank(),
        panel.background = element_blank(), 
        axis.line = element_line(colour = "black"))


#Now let's make a two-panel picture


figure3 <- ggarrange(predictionsA, predictionsB, 
                     vjust = 4, hjust = 4, font.label = list(size = 8),
                     ncol = 1, nrow = 2)
annotate_figure(figure3,
                top = text_grob("Figure 3 - Pre-1950 War Patterns Predict more Post-1950 Wars of all Sizes than Actual", 
                                vjust = 2.5, size = 12))


ggsave("graph3.png",device = "png" ,dpi = 700, width = 7.5, height = 10, units = "in")




#Next we assess the power of the tests built into graph 3

power <- c()
for (i in c(0.5,0.6,0.7,0.8,0.9, 1.0)){
  for (j in 61*seq(0.05, 1.5, 0.05)) {
    power <- c(power, ppois(qpois(0.025, j)-1,i*j))
  }
}

power_df <- data.frame(matrix(power, nrow = 30, ncol = 6))

rates <- seq(0.05, 1.5, 0.05)

#Calculate the arrival rate for wars above the median size
rate_median <- (252/2)/(2011-1799)

power_picture  <- ggplot(power_df, aes(x= rates))+
  geom_line(aes(y = X1)) +
  geom_line(aes(y = X2)) +
  geom_line(aes(y = X3)) +
  geom_line(aes(y = X4)) +
  geom_line(aes(y = X5)) +
  geom_line(aes(y = X6)) +
  geom_vline(xintercept = rate_median, linetype = "dotted")+
  geom_text(x = 0.7, y = 0.97, label = "0.5", cex = 3)+
  geom_text(x = 0.7, y = 0.81, label = "0.6", cex = 3)+
  geom_text(x = 0.7, y = 0.51, label = "0.7", cex = 3)+
  geom_text(x = 0.7, y = 0.24, label = "0.8", cex = 3)+
  geom_text(x = 0.7, y = 0.09, label = "0.9", cex = 3)+
  geom_text(x = 0.7, y = 0.03, label = "1.0", cex = 3)+
  geom_text(x = 0.71, y = 1.02, label = "median arrival rate", cex = 3)+
  labs(title = "Figure 4 - The Tests Have Low Power for Detecting Slowdowns",
       y = "Probability of Clearing a Statistical Significance Hurdle", 
       x = "War Arrival Rates") + 
  theme(plot.title = element_text(size=12)) +
  theme(plot.subtitle = element_text(size = 10)) +
  theme(panel.grid.major = element_blank(), 
        panel.grid.minor = element_blank(),
        panel.background = element_blank(), 
        axis.line = element_line(colour = "black"))



ggsave("graph4.png",device = "png" ,dpi = 700, width = 7.5, height = 10, units = "in")

#The conclusion is that when the size range starts much above the median war size
#for the whole Mars dataset then the probability of detecting a real decrease
#in arrival rates, post versus pre 1950 is low unless the rate difference is really
#large, e.g., 40 or 50 percent.


#Now let's redo figure 3 but using figures per 100,000 of world population


#Create a vector giving the number of wars exceeding the size of each post-1950 war
number_over_low_ph <- c()
for(i in 1:252) {
  if(warsizes[i,"startyear"] > 1950) {
    temporary <- warsizes[i, "kialow_ph"]
    number_over_low_ph <- c(number_over_low_ph, 
                            nrow(filter(warsizes, startyear < 1951, kialow_ph > temporary)) )
  }
}



#The idea here is that for each post-1950 war you calculate the number of 
#post-1950 wars there should be at or above that size if post-1950 wars in 
#that size range arrive at the rate they did before 1950

number_low_post_ph <- c(1:length(number_over_low_ph))
number_over_low_ph_sorted <- sort(number_over_low_ph)
pred_low_ph_central <- (number_over_low_ph_sorted)*61/151
pred_low_ph_upper  <- qpois(.975, pred_low_ph_central, lower.tail = TRUE)
pred_low_ph_lower  <- qpois(.025, pred_low_ph_central, lower.tail = TRUE)
pred_low_ph_lower_.05 <- qpois(.05, pred_low_ph_central, lower.tail = TRUE)
pred_low_ph_lower_.10 <- qpois(.10, pred_low_ph_central, lower.tail = TRUE)


kia_low_post_ph <- warsizes %>% filter(startyear >1950)%>%select(kialow_ph)
temp  <- arrange(kia_low_post_ph, kialow_ph)
kia_low_post_ph_asc <- temp$kialow_ph
number_low_post_ph_desc <- sort(number_low_post_ph, decreasing = TRUE)
pred_low_ph_central_desc <- sort(pred_low_ph_central, decreasing = TRUE)
pred_low_ph_upper_desc <- sort(pred_low_ph_upper, decreasing = TRUE)
pred_low_ph_lower_desc <- sort(pred_low_ph_lower, decreasing = TRUE)
pred_low_ph_lower_.05_desc <- sort(pred_low_ph_lower_.05, decreasing = TRUE)
pred_low_ph_lower_.10_desc <- sort(pred_low_ph_lower_.10, decreasing = TRUE)


cardinal_df_low <- data.frame(kia_low_post_ph_asc,number_low_post_ph_desc,
                              pred_low_ph_central_desc, pred_low_ph_lower_desc,
                              pred_low_ph_upper_desc)



predictions_ph_A <- ggplot(cardinal_df_low, aes(x= kia_low_post_ph_asc ))+
  geom_line(aes(y = pred_low_ph_central_desc),linetype = "dotted") +
  geom_line(aes(y = number_low_post_ph_desc)) +
  geom_line(aes(y = pred_low_ph_lower_desc), linetype="dashed") +
  geom_line(aes(y = pred_low_ph_lower_.10_desc), linetype="dashed") +
  scale_x_continuous(trans='log10') +
  labs(x = "Mars Low Estimates of Killed in Action per 100,000",
       y = "Predicted and Actual Counts of Post-1950 Wars") + 
  annotate(geom = "text", x = 0.075, y = 75, label = "predicted number of wars",
           size = 2)+
  annotate(geom = "text", x = 0.017, y = 71.5, label = "lower limit (0.1)",
           size = 2)+
  annotate(geom = "text", x = 0.015, y = 60, label = "lower limit (0.025)",
           size = 2)+
  annotate(geom = "text", x = 0.05, y = 27, label = "actual number of wars",
           size = 2)+
  theme(panel.grid.major = element_blank(), 
        panel.grid.minor = element_blank(),
        plot.margin = margin(t = 10, r = 4, b = 4, l = 7, unit = "pt"),
        panel.background = element_blank(), 
        axis.line = element_line(colour = "black"))

#Now we do the same thing but for the high estimates of KIA

#Create a vector giving the number of wars exceeding the size of each post-1950 war
number_over_high_ph <- c()
for(i in 1:252) {
  if(warsizes[i,"startyear"] > 1950) {
    temporary <- warsizes[i, "kiahigh_ph"]
    number_over_high_ph <- c(number_over_high_ph, 
                             nrow(filter(warsizes, startyear < 1951, kiahigh_ph > temporary)) )
  }
}


#The idea here is that for each post-1950 war you calculate the number of 
#post-1950 wars there should be at or above that size if post-1950 wars in 
#that size range arrive at the rate they did before 1950

number_high_post_ph <- c(1:length(number_over_high_ph))
number_over_high_ph_sorted <- sort(number_over_high_ph)
pred_high_ph_central <- (number_over_high_ph_sorted)*61/151
pred_high_ph_upper  <- qpois(.975, pred_high_ph_central, lower.tail = TRUE)
pred_high_ph_lower  <- qpois(.025, pred_high_ph_central, lower.tail = TRUE)
pred_high_ph_lower_.05 <- qpois(.05, pred_high_ph_central, lower.tail = TRUE)
pred_high_ph_lower_.10 <- qpois(.10, pred_high_ph_central, lower.tail = TRUE)



kia_high_post_ph <- warsizes %>% filter(startyear >1950)%>%select(kiahigh_ph)
temp  <- arrange(kia_high_post_ph, kiahigh_ph)
kia_high_post_ph_asc <- temp$kiahigh_ph
number_high_post_ph_desc <- sort(number_high_post_ph, decreasing = TRUE)
pred_high_ph_central_desc <- sort(pred_high_ph_central, decreasing = TRUE)
pred_high_ph_upper_desc <- sort(pred_high_ph_upper, decreasing = TRUE)
pred_high_ph_lower_desc <- sort(pred_high_ph_lower, decreasing = TRUE)
pred_high_ph_lower_.05_desc <- sort(pred_high_ph_lower_.05, decreasing = TRUE)
pred_high_ph_lower_.10_desc <- sort(pred_high_ph_lower_.10, decreasing = TRUE)


cardinal_df_high <- data.frame(kia_high_post_ph_asc,number_high_post_ph_desc,
                               pred_high_ph_central_desc, pred_high_ph_lower_desc,
                               pred_high_ph_upper_desc)



predictions_ph_B <- ggplot(cardinal_df_high, aes(x= kia_high_post_ph_asc ))+
  geom_line(aes(y = pred_high_ph_central_desc),linetype = "dotted") +
  geom_line(aes(y = number_high_post_ph_desc)) +
  geom_line(aes(y = pred_high_ph_lower_desc), linetype="dashed") +
  geom_line(aes(y = pred_high_ph_lower_.10_desc), linetype="dashed") +
  scale_x_continuous(trans='log10') +
  labs(x = "Mars high Estimates of Killed in Action per 100,000",
       y = "Predicted and Actual Counts of Post-1950 Wars") + 
  theme(plot.title = element_text(size=12)) +
  theme(plot.subtitle = element_text(size = 10)) +
  annotate(geom = "text", x = 0.205, y = 72, label = "predicted number of wars",
           size = 2)+
  annotate(geom = "text", x = 0.02, y = 71, label = "lower limit (0.1)",
           size = 2)+
  annotate(geom = "text", x = 0.018, y = 61, label = "lower limit (0.025)",
           size = 2)+
  annotate(geom = "text", x = 0.1, y = 27, label = "actual number of wars",
           size = 2)+
  theme(panel.grid.major = element_blank(), 
        panel.grid.minor = element_blank(),
        plot.margin = margin(t = 4, r = 4, b = 4, l = 7, unit = "pt"),
        panel.background = element_blank(), 
        axis.line = element_line(colour = "black"))


figure5 <- ggarrange(predictions_ph_A, predictions_ph_B, 
                      vjust = 4, font.label = list(size = 8),
                      ncol = 1, nrow = 2)
annotate_figure(figure5,
                top = text_grob("Figure 5 - Pre-1950 War Patterns Predict more Post-1950 Wars of all Sizes than Actually Happen", 
                                vjust = 2.5, size = 12))


ggsave("graph5.png", device = "png" ,dpi = 700, width = 7.5, height = 10, units = "in")




#The following table and figures are in the appendices

#We start with Appendix 2 which is a critique of the critiques of the 
#decline of war theses

#Table 1Appendix gives simulation results for which which we take draws from power laws 
#with various exponents and an xmin of 1,000 and then estimate power-fits from
#the simulated data. The point is to understand the distribution of the errors
#in these estimates. Here is the data for this exercise

alphas_1000 <- read.csv("alphas_1000.csv")
pl_data_1000 <- read.csv("pl_data_1000.csv")

#The next three datasets was produced by us using several packages and the CoW dat.  
#It takes a while to run so we make the final product available here. They give 
#fits using power law, expontential and generalized gamma distributions to,
#respectively, the CoW, Mars (low estimates) and Mars (high estimates) data

fits_all_CoW <- read.csv("fits_all_CoW.csv")

fits_all_low <- read.csv("fits_all_low.csv")

fits_all_high <- read.csv("fits_all_high.csv")

#We now produce Table 1A of the appendix showing that estimates of power laws can
#easily get the estimates wrong in ways that are consequential for the decline
#of war debate

prob_alphas_6_6 <- sum(alphas_1000$X1.6 < 1.6)/200
prob_alphas_7_6 <- sum(alphas_1000$X1.7 < 1.6)/200
prob_alphas_8_6 <- sum(alphas_1000$X1.8 < 1.6)/200
prob_alphas_9_6 <- sum(alphas_1000$X1.9 < 1.6)/200

prob_alphas_6_7 <- sum(alphas_1000$X1.6 < 1.7)/200
prob_alphas_7_7 <- sum(alphas_1000$X1.7 < 1.7)/200
prob_alphas_8_7 <- sum(alphas_1000$X1.8 < 1.7)/200
prob_alphas_9_7 <- sum(alphas_1000$X1.9 < 1.7)/200

prob_1.5_100000 <- sum(pl_data_1000$X1.5 > 100000)/nrow(pl_data_1000)
prob_1.6_100000  <- sum(pl_data_1000$X1.6 > 100000)/nrow(pl_data_1000)
prob_1.7_100000  <- sum(pl_data_1000$X1.7 > 100000)/nrow(pl_data_1000)
prob_1.8_100000  <- sum(pl_data_1000$X1.8 > 100000)/nrow(pl_data_1000)
prob_1.9_100000  <- sum(pl_data_1000$X1.9 > 100000)/nrow(pl_data_1000)

ratio_100000_1.6 <- prob_1.5_100000/prob_1.6_100000
ratio_100000_1.7 <- prob_1.5_100000/prob_1.7_100000
ratio_100000_1.8 <- prob_1.5_100000/prob_1.8_100000
ratio_100000_1.9 <- prob_1.5_100000/prob_1.9_100000


prob_1.5_1000000 <- sum(pl_data_1000$X1.5 > 1000000)/nrow(pl_data_1000)
prob_1.6_1000000 <- sum(pl_data_1000$X1.6 > 1000000)/nrow(pl_data_1000)
prob_1.7_1000000 <- sum(pl_data_1000$X1.7 > 1000000)/nrow(pl_data_1000)
prob_1.8_1000000 <- sum(pl_data_1000$X1.8 > 1000000)/nrow(pl_data_1000)
prob_1.9_1000000 <- sum(pl_data_1000$X1.9 > 1000000)/nrow(pl_data_1000)

ratio_1000000_1.6 <- prob_1.5_1000000/prob_1.6_1000000
ratio_1000000_1.7 <- prob_1.5_1000000/prob_1.7_1000000
ratio_1000000_1.8 <- prob_1.5_1000000/prob_1.8_1000000
ratio_1000000_1.9 <- prob_1.5_1000000/prob_1.9_1000000

prob_1.5_10000000 <- sum(pl_data_1000$X1.5 > 10000000)/nrow(pl_data_1000)
prob_1.6_10000000 <- sum(pl_data_1000$X1.6 > 10000000)/nrow(pl_data_1000)
prob_1.7_10000000 <- sum(pl_data_1000$X1.7 > 10000000)/nrow(pl_data_1000)
prob_1.8_10000000 <- sum(pl_data_1000$X1.8 > 10000000)/nrow(pl_data_1000)
prob_1.9_10000000 <- sum(pl_data_1000$X1.9 > 10000000)/nrow(pl_data_1000)

ratio_10000000_1.6 <- prob_1.5_10000000/prob_1.6_10000000
ratio_10000000_1.7 <- prob_1.5_10000000/prob_1.7_10000000
ratio_10000000_1.8 <- prob_1.5_10000000/prob_1.8_10000000
ratio_10000000_1.9 <- prob_1.5_10000000/prob_1.9_10000000

numbers <- c(prob_alphas_6_6, prob_alphas_7_6, prob_alphas_8_6, prob_alphas_9_6,
             prob_alphas_6_7, prob_alphas_7_7, prob_alphas_8_7, prob_alphas_9_7,
             ratio_100000_1.6, ratio_100000_1.7, ratio_100000_1.8, ratio_100000_1.9,
             ratio_1000000_1.6, ratio_1000000_1.7, ratio_1000000_1.8, 
             ratio_1000000_1.9,
             ratio_10000000_1.6, ratio_10000000_1.7, ratio_10000000_1.8, 
             ratio_10000000_1.9)

simulation_table <- as.data.frame(t(matrix(numbers, 4, 5)))
names(simulation_table)[1:4] <- c("1.6", "1.7", "1.8", "1.9")
situations <- c("P(alpha estimate < 1.6)","P(alpha estimate < 1.7)",
                "P(x > 100,000|alpha = 1.5)/P(x > 100,000)",
                "P(x > 1,000,000|alpha = 1.5)/P(x > 1,000,000)",
                "P(x > 10,000,000|alpha = 1.5)/P(x > 10,000,000)")
simulation_table <- cbind(situations, simulation_table)

table1Appendix <- simulation_table %>%
  gt() %>%
  tab_header(
    title = "Table 1A - Power Law Estimates can Easily Overestimate Tail Probabilities Substantially",
  )%>%
  cols_label(situations = " ")%>%
  tab_spanner(
    label = "True Alphas",
    columns = c(2:5))%>%
  tab_row_group(
    label = "Probabilities of Underestimating Alpha",
    rows = 1:2
  ) %>%
  tab_row_group(
    label = "Relative Tail Probabilities",
    rows = 3:5
  )%>%
  row_group_order(groups = c("Probabilities of Underestimating Alpha", 
                             "Relative Tail Probabilities"))%>%
  tab_source_note(
    source_note = "Simulated Data")%>%
  fmt_number(c(2:5),decimals = 2, sep_mark = "," )

gtsave(table1Appendix, "table1Appendix.png")




#Next is the code for the graph in Appendix 2

post_1945_CoW <- filter(cow, startyear > 1944)$deaths


plot1 <- fits_all_CoW%>%
  ggplot(aes(x = x, y = y))+
  geom_point(data = filter(fits_all_CoW,cat == "kiaCoW")) +
  geom_line(data = filter(fits_all_CoW,cat == "pl")) +
  geom_line(data = filter(fits_all_CoW,cat == "ln"), linetype = "dashed")+
  geom_line(data = filter(fits_all_CoW,cat == "gg"), linetype = "dotted")+
  geom_vline(xintercept = 6525, color = "grey")+
  scale_x_continuous(trans='log10') +
  scale_y_continuous(trans='log10') +
  geom_text(aes(x = 5000000, y = 0.01),label = "lognormal", cex = 2) +
  geom_text(aes(x = 7800000, y = 0.015),label = "power law", cex = 2) +
  geom_text(aes(x = 13500000, y = 0.025),label = "Generalized Gamma", cex = 2) +
  geom_text(aes(x = 7500, y = 0.01),label = "xmin", cex = 2) +
  labs(title = "The Graph Shows All CoW Inter-State Data Points",
       y = "Prob(X > x)", 
       x = "Battle Death Counts")+
  theme(panel.grid.major = element_blank(), 
        panel.grid.minor = element_blank(),
        panel.background = element_blank(), 
        axis.line = element_line(colour = "black"),
        plot.margin = unit(c(1.0,0.1,0.1,0), "cm"))

plot2 <- fits_all_CoW%>%
  ggplot(aes(x = x, y = y))+
  geom_point(data = filter(fits_all_CoW,cat == "kiaCoW" & x %in% post_1945_CoW))+
  geom_line(data = filter(fits_all_CoW,cat == "pl")) +
  geom_line(data = filter(fits_all_CoW,cat == "ln"), linetype = "dashed")+
  geom_line(data = filter(fits_all_CoW,cat == "gg"), linetype = "dotted")+
  geom_vline(xintercept = 6525, color = "grey")+
  scale_x_continuous(trans='log10') +
  scale_y_continuous(trans='log10') +
  geom_text(aes(x = 5000000, y = 0.01),label = "lognormal", cex = 2) +
  geom_text(aes(x = 7800000, y = 0.015),label = "power law", cex = 2) +
  geom_text(aes(x = 13500000, y = 0.025),label = "Generalized Gamma", cex = 2) +
  geom_text(aes(x = 7500, y = 0.01),label = "xmin", cex = 2) +
  labs(title = "This Graph Shows Only Post-1945 CoW Inter-State Data Points",
       subtitle = "The curves are still fit to all the data points as in panel a",
       y = "Prob(X > x)", 
       x = "Battle Death Counts")+
  theme(panel.grid.major = element_blank(), 
        panel.grid.minor = element_blank(),
        panel.background = element_blank(), 
        axis.line = element_line(colour = "black"),
        plot.margin = unit(c(1.0,0.1,0.1,0), "cm"))


plot3 <- fits_all_CoW%>%
  ggplot(aes(x = x, y = y))+
  geom_point(data = filter(fits_all_CoW,cat == "kiaCoW" & x %in% post_1945_CoW))+
  geom_line(data = filter(fits_all_CoW,cat == "pl")) +
  geom_line(data = filter(fits_all_CoW,cat == "ln"), linetype = "dashed") +
  geom_line(data = filter(fits_all_CoW,cat == "gg"), linetype = "dotted")+
  #scale_x_continuous(trans='log10') +
  scale_y_continuous(trans='log10') +
  geom_text(aes(x = 4550000, y = 0.0115),label = "lognormal", cex = 2) +
  geom_text(aes(x = 8000000, y = 0.0135),label = "power law", cex = 2) +
  geom_text(aes(x = 13500000, y = 0.0225),label = "Generalized Gamma", cex = 2) +
  labs(title = "This Graph is the same as panel b but with an unlogged x axis",
       y = "Prob(X > x)", 
       x = "Battle Death Counts")+
  theme(panel.grid.major = element_blank(), 
        panel.grid.minor = element_blank(),
        panel.background = element_blank(), 
        axis.line = element_line(colour = "black"),
        plot.margin = unit(c(1.0,0.1,0.1,0), "cm"))


#Put the two above plots into a three-panel picture

figure1Appendix <- ggarrange(plot1, plot2, plot3,
                     vjust = 4, font.label = list(size = 8),
                     labels = c("a", 
                                "b",
                                "c"
                     ),
                     ncol = 1, nrow = 3)
figure1Appendix <- annotate_figure(figure1Appendix,
                           fig.lab = "Figure 1A - Three Fits to CoW Interstate Battle Death Counts",
                           fig.lab.size = 14
)

ggsave("graph1Appendix.png", device = "png" ,dpi = 700, width = 7.5, height = 10, units = "in")

#For an appendix of alternative exploratory graphs


#The raw time series
raw_plota <- annual_totals %>%
  ggplot(aes(x = startyear, y = kialow)) +
  geom_line() +
  labs(title = "Raw Annual Time Series", 
       y = "KIA (low)", x = "Year") + 
  theme(panel.grid.major = element_blank(), 
        panel.grid.minor = element_blank(),
        panel.background = element_blank(), 
        axis.line = element_line(colour = "black"),
        plot.margin = unit(c(1,0.2,0.2,0.2), "cm"))

#The world wars dominate to the point where the picture isn't very interesting

#logging the y axis doesn't seem to produce much insight
log_plota <- annual_totals %>%
  ggplot(aes(x = startyear, y = kialow)) +
  geom_line() +
  scale_y_log10() +
  labs(title = "Annual Deaths with a Logged Y Axis", 
       y = "KIA (low)", x = "Year") + 
  theme(panel.grid.major = element_blank(), 
        panel.grid.minor = element_blank(),
        panel.background = element_blank(), 
        axis.line = element_line(colour = "black"))

#Note - getting an infinite values error message.  This should be fixable by 
#adding 1 to all the KIALOW numbers but R seems to be handling it fine anyway.

#Conclusion - the world wars dominate the picture so much that you can't put 
#in horizontal lines for averages that don't include the world wars as these
#horizontal lines will hug the x axis

#here's a moving average
moving_average <- annual_totals %>%
  ggplot(aes(x = startyear, y = kialow)) +
  geom_ma(ma_fun = SMA, n = 30,linetype = "solid", colour = "black")+
  labs(title = "Thirty Year Moving Average of Annual Deaths", 
       y = "KIA (low)", x = "Year") + 
  theme(panel.grid.major = element_blank(), 
        panel.grid.minor = element_blank(),
        panel.background = element_blank(), 
        axis.line = element_line(colour = "black"))


#Let's put together a 3-panel picture for the paper



figure2Appendix <- ggarrange(raw_plota, log_plota, moving_average,
                      vjust = 4, font.label = list(size = 8),
                      labels = c("a", 
                                 "b",
                                 "c"
                      ),
                      ncol = 1, nrow = 3)
figure2Appendix  <- annotate_figure(figure2Appendix ,
                            fig.lab = "Figure 2A - Time Series Graphs of Mars KIA Data",
                            fig.lab.size = 14
)

ggsave("graph2Appendix.png", device = "png" ,dpi = 700, width = 7.5, height = 10, units = "in")

#Let's do the same picture for KIAHIGH for the appendix

#The raw time series
raw_plot_a <- warsizes %>%
  ggplot(aes(x = startyear, y = kiahigh)) +
  geom_point() +
  labs(title = "The Start Year Versus the KIA Count for Each War", 
       y = "KIA (high)", x = "Year") + 
  theme(panel.grid.major = element_blank(), 
        panel.grid.minor = element_blank(),
        panel.background = element_blank(), 
        axis.line = element_line(colour = "black"),
        plot.margin = unit(c(0.8,0.2,0.2,0.2), "cm"))



#This is visually the same as the old one

#The world wars dominate to the point where the picture isn't very interesting

#logging the y axis doesn't seem to produce much insight
log_plot_a <- warsizes %>%
  ggplot(aes(x = startyear, y = kiahigh)) +
  geom_point() +
  scale_y_log10() +
  labs(title = "The Start Year Versus the KIA Count for Each War",
       subtitle = "Logged Y Axis", 
       y = "KIA (high)", x = "Year") + 
  theme(panel.grid.major = element_blank(), 
        panel.grid.minor = element_blank(),
        panel.background = element_blank(), 
        axis.line = element_line(colour = "black"))

#again there's no visual change from the earlier picture



distribution_a <- fits_all_high%>%
  ggplot(aes(x = x, y = y))+
  geom_point(data = filter(fits_all_high, cat == "kiahigh"))+
  scale_y_continuous(trans='log10') +
  scale_x_continuous(trans='log10') +
  labs(title = "The Distribution of KIA Counts on log-log Axes",
       y = "Prob(X > x)", 
       x = "Battle Death Counts")+
  theme(panel.grid.major = element_blank(), 
        panel.grid.minor = element_blank(),
        panel.background = element_blank(), 
        axis.line = element_line(colour = "black"),
        plot.margin = unit(c(0.1,0.1,0.1,1.00), "cm")
  )

#Now put it together in a single figure

figure3Appendix <- ggarrange(raw_plot_a, log_plot_a, distribution_a, 
                      vjust = 4, font.label = list(size = 8),
                      labels = c("a", 
                                 "b",
                                 "c"
                      ),
                      ncol = 1, nrow = 3)
figure3Appendix <- annotate_figure(figure3Appendix,
                            fig.lab = "Figure 3A - Three Views of the Mars KIA Counts for all Wars",
                            fig.lab.size = 14
)

ggsave("graph3Appendix.png", device = "png" ,dpi = 700, width = 7.5, height = 10, units = "in")

#Note - Do these pictures in the appendix for KIA per 100,000 of world population
