#
# This is a Shiny web application. You can run the application by clicking
# the 'Run App' button above.
#
# Find out more about building applications with Shiny here:
#
#    http://shiny.rstudio.com/
#

#TODO update the discussion in the text.

# Libraries: ###################################################################
if (!require("rethinking")) {
  caninstall <- readline(
    prompt =
      "Rethinking by McElreath not installed; minimal install from github now? (Y/N)"
  )
  if (toupper(caninstall) == "Y" && require("devtools")) {
    devtools::install_github("rmcelreath/rethinking@slim")
  } else if (!require("devtools")) {
    print("devtools not detected. We use this to install from github.")
    print("Please install devtools in the typical way.")
    print("install.packages('devtools')")
    stop()
  } else {
    print("Y not detected, quitting.")
    stop()
  }
}

library(shiny)
library(shinyWidgets)
library(rethinking)
library(dplyr)
library(tidyr)

# Set-up: ######################################################################

# From Michael Spagat.
warsizes <- read.csv(file.path(".", "warsizes.csv"))

#TODO Can be fixed more appropriately later.
warsizes$startdate <- as.Date(paste0(warsizes$startyear, "-01-01"))

firstwar <- warsizes[which.min(warsizes$startdate),]
lastwar <- warsizes[which.max(warsizes$startdate),]

minkialow <- min(warsizes$kialow)
minkialow_ph <- min(warsizes$kialow_ph)
maxkiahigh <- max(warsizes$kiahigh)
maxkiahigh_ph <- max(warsizes$kiahigh_ph)

# Functions: ###################################################################

defFormat <- function(x) formatC(x, digits = 4)
defSignif <- function(x) signif(x, digits = 4)

updateExpectedPrior <- function(
    IN_kiaThreshold = NULL, #c(input$IN_kiaThreshold1, input$IN_kiaThreshold2),
    IN_datesPosterior = NULL, #input$IN_datesPosterior
    IN_highlow = "Low", IN_perCap = "Raw"
) {

  if (is.null(IN_kiaThreshold)) {
    IN_kiaThreshold <- c(0, Inf)
  }

  if (is.null(IN_datesPosterior)) {
    IN_datesPosterior <- lastwar$startdate
  }

  ExpPrior <- warsizes

  if (IN_highlow == "Low" && IN_perCap == "Raw") {
    ExpPrior$Size <- ExpPrior$kialow
  }
  if (IN_highlow == "High" && IN_perCap == "Raw") {
    ExpPrior$Size <- ExpPrior$kiahigh
  }
  if (IN_highlow == "Low" && IN_perCap == "Per Capita") {
    ExpPrior$Size <- ExpPrior$kialow_ph
  }
  if (IN_highlow == "High" && IN_perCap == "Per Capita") {
    ExpPrior$Size <- ExpPrior$kiahigh_ph
  }

  ExpPrior <- ExpPrior %>% dplyr::filter(
    Size >= IN_kiaThreshold[1],
    Size <= IN_kiaThreshold[2],
    startdate < IN_datesPosterior[1]
  ) %>% dplyr::group_by(
    startyear
  ) %>% dplyr::summarise(
    WarCounts = dplyr::n(), .groups = "drop"
  ) %>% tidyr::complete(
    startyear = seq(as.numeric(strtrim(firstwar$startdate, 4)),
                    as.numeric(strtrim(IN_datesPosterior[1], 4)) - 1,
                    # Don't include the end (if it doesn't have any wars).
                    1),
    fill = list(WarCounts = 0)
  ) %>% dplyr::summarise(
    Number = sum(WarCounts),
    Years = length(WarCounts),
    Mean = Number / Years,
    StdDev = sqrt(Number / (Years^2))
  )

  return(ExpPrior)
}

# UI: ##########################################################################
# Define UI for application that draws a histogram
ui <- fluidPage(

  # Application title
  titlePanel("Mars: Bayesian Arrival Rates"),

  tabsetPanel(
    tabPanel(
      "Description",
      strong("Introduction:"),
      p("Welcome to our tool for the analysis of war arrival rates using the Mars
        dataset."),
      p("The first thing you should see on the 'Main' tab offers you a choice of seed
      for your randomizations. You can safely just leave this set to 1 or you can
      change it and get slightly different randomization outcomes.
      Ignore, for the moment, the 'calibrate prior', 'compute posterior' and
      'Save plot' buttons. Instead, push the 'Data' button."),
      strong("Data:"),
      p("First you choose the time period covered by the posterior. Next is your
        KIA measure which can be one of four possibilities: high or low estimates
        and raw counts or counts per 100,000 of world population. Finally, you
        specify the range of war sizes (maximum and mimimum) under consideration.
        Now push the 'Distributions' button."),
      strong("Distributions:"),
      p("All but one of the choices under the distributions button concern the
        choice of prior so we explain these choices first."),
      p("First choose either a gamma, normal or uniform prior."),
      p("If you choose a gamma prior, i.e., the conjugate prior,
        then the system automatically offers you a default option with the mean
        and the standard deviation set equal to their empirical values for the
        size range and period that you have chosen previously. You can then
        manually tweak the gamma parameters away from these default values, using
        a menu that presents the parameter choices in terms of an 'effective
        number of wars' and 'effective number of years'. At the default setting
        these are set equal to their empirical values but you can, e.g., increase
        the mean of the prior on the arrival rate by increasing the effective
        number of wars and/or decreasing the effective number of years. A possible
        basis for such tweaking could be a belief that a single war listed in
        the Mars database should be separated into multiple wars or vice versa.
        The text at the bottom of the screen gives the resulting mean and standard deviation
        of the prior."),
      p("When you select a normally distributed prior the system, again, sets
        default mean and standard deviation values equal to their empirical rates
        but you can then tweak your way away from these values.
        Text at the bottom of the screen converts your parameter values
        to the equivalent effective numbers of wars and years.
        This should allow you to use whichever is easier or to move between the
        two distributions.
        (The conversion uses the gamma distribution's relationships as an approximation.)"),
      p("The final prior choice on the menu is the uniform, which is treated
        differently from the gamma and normal choices in that the default setting,
        a uniform distribution between 0 and 7, is largely uninformed by the
        data aside from the facts that 0 is the minimum and 7 is the maximum
        number of wars occurring within a single year in the Mars data.
        The uniform distribution can be used to exclude posterior arrival rates
        by not including them between the minimum and maximum, but is heavily
        reliant on the data being fitted. This can result in an error
        if your prior rules out arrival rates that are observed in the data."),
      p("We sound two notes of caution regarding the uniform distribution, i.e.,
        'flat priors'. First, these are not usually recommended because,
        typically, the user has some outside beliefs or knowledge that can and
        should be included in a Bayesian analysis. Second, due to the quadratic
        approximation that is employed to speed computation, some arrival rates
        that should be excluded by a uniform prior can still be observed due to
        the quadratic smoothing. This is just a minor technical issue and would
        not happen with a fuller (but more computationally intensive) analysis."),
      p("The above choices provide users of this tool with considerable flexibility
        in choices of prior distributions. We note, however, that the default
        choices for the gamma and normal distributions are stongly based on the
        data so you should at least try to articulate reasons for departing from
        them if you do."),
      p("We also note for context that",
        tags$a(href = 'https://github.com/stan-dev/stan/wiki/Prior-Choice-Recommendations',
               "STAN"),
        "suggests that one should use a fairly weak prior so as to allow",
        "for the possibility of being surprised.",
        tags$br(),tags$br()),
      p("The final choice under the distributions button is for the width of the
        central credibility interval for the posterior. The default is for a 50%
        credibility interval."),
      strong("The Remaining Buttons:"),
      p("The 'Calibrate Prior' button takes you back to the default, i.e., data
        based, parameters for gamma or normal priors. You can use it if, e.g,
        you have tweaked a normal or gamma prior, want to return to the default
        setting but have forgotten what it is."),
      p("The 'Compute Posterior' button starts the workhorse, powered by McElreath's
        rethinking package. Only push it after you are satisfied with your
        choice of data and prior. The main panel will then return the mean,
        standard deviation, credibility interval, a small histogram, and the
        probability that the 'posterior' arrival rate is smaller than your
        prior's arrival rate, alongside a plot. The plot visualise how the rate
        has changed from prior (blue) to posterior (black). The value of the
        credibility interval in the text is fixed to the value used when
        'Compute Posterior' was pressed."),
      p("Of course, the 'Save Plot' button saves your plot"),
      strong("Reading the Graphs:"),
      p("The posterior distribution is drawn in black while the prior distribution
        is drawn in blue. The mean for the prior is shown with a vertical line
        with blue dashes. The 'historical mean' (red dots) is the empirical
        arrival rate of wars between 1800 and your lower choice of 'Dates in
        posterior'. If you accept the default parameters for the gamma and
        normal priors, the historical mean will be identical to the
        means for your prior. However, if you tweak the prior away from the default
        parameter values then the mean of the prior will diverge from the
        historical mean. The 'Post Data Mean' is the empirical arrival rate
        during the period covered by the posterior."),

    ),

    tabPanel(
      "Main",
      # Sidebar with a slider input for number of bins
      sidebarLayout(
        sidebarPanel(
          shiny::numericInput(
            "IN_seed",
            label = "Random Seed",
            value = 1
          ),

          shiny::actionButton(
            "IN_currentDefault",
            label = "Calibrate Prior"
          ),

          shiny::actionButton(
            "IN_startComputation",
            label = "Compute Posterior"
          ),

          shiny::downloadButton(
            "downloadImage",
            label = "Save Plot"
          ),

          shiny::tabsetPanel(
            shiny::tabPanel(
              "Data",

              shiny::dateRangeInput(
                "IN_datesPosterior",
                "Dates in Posterior",
                start = "1950-01-01",
                end = "2011-12-31",
                min = "1800-01-01",
                max = "2011-12-31",
                startview = "year"
              ),

              shiny::radioButtons(
                "IN_highlow",
                "High KIA or Low KIA?",
                choices = c(
                  "High", "Low"
                ),
                selected = "Low"
              ),

              shiny::radioButtons(
                "IN_perCap",
                "Raw KIA or KIA Per Capita?",
                choices = c(
                  "Raw", "Per Capita"
                ),
                selected = "Raw"
              ),

              shiny::fluidPage(
                shiny::uiOutput(
                  # Depends on IN_perCap, provides IN_kiaThreshold.
                  "OUT_kiaThreshold"
                )
              )
            ),
            shiny::tabPanel(
              "Distribution",


              shiny::selectInput(
                "IN_distribution",
                "Choice of prior distribution",
                multiple = FALSE,
                choices = c(
                  "gamma", "normal", "uniform"
                ),
                selected = "gamma"
              ),

              shiny::uiOutput(
                "OUT_par1"
              ),

              shiny::uiOutput(
                "OUT_par2"
              ),

              shiny::uiOutput(
                "OUT_priorSummary"
              ),
              p(""),
              shiny::numericInput(
                "IN_ciWidth",
                "Width of the Central Credibility Interval",
                min = 0.01, max = 0.99, value = 0.5
              ),
            )
          )

        ),

        # Show a plot of the generated distribution
        mainPanel(
          shiny::textOutput("OUT_QuapPars1"),
          shiny::textOutput("OUT_QuapPars2"),
          shiny::plotOutput("OUT_QuapPlot"),
          shiny::textOutput("OUT_Error")
        )
      )
    )
  )
)

# Server: ######################################################################

server <- function(input, output, session) {

  ### Reactive UI: #############################################################
  output$OUT_kiaThreshold <- shiny::renderUI({
    mini <- switch(input$IN_perCap,
                   "Raw" = minkialow,
                   "Per Capita" = floor(minkialow_ph/10^(-4))*10^-4,
                   0
    )
    maxi <- switch(input$IN_perCap,
                   "Raw" = ceiling(maxkiahigh/10^6)*10^6,
                   "Per Capita" = ceiling(maxkiahigh_ph/10^2)*10^2,
                   1e10
    )

    shiny::fluidPage(
      shiny::strong("KIA Thresholds:"),
      shiny::numericInput(
        "IN_kiaThreshold1",
        "Minimum",
        min = mini,
        max = maxi,
        value = c(mini)
      ),

      shiny::numericInput(
        "IN_kiaThreshold2",
        "Maximum",
        min = mini,
        max = maxi,
        value = c(maxi)
      )
    )
  })

  ### Reactive Backend: ########################################################
  TEMP <- shiny::reactiveValues(
    # EXPected Prior.
    ExpPrior = data.frame(
      Number = NA, Years = NA, Mean = NA, StdDev = NA
    ),
    # Quadratic Approximation
    Quap = NA, Error = NA,
    # Results of a run in case of plotting.
    Results = NA, ResultsFlag = NA
  )

  # Update Expected Prior if Distribution Changes.
  shiny::observeEvent(input$IN_distribution, {
    TEMP$ExpPrior <- updateExpectedPrior(
      IN_kiaThreshold = c(input$IN_kiaThreshold1, input$IN_kiaThreshold2),
      IN_datesPosterior = input$IN_datesPosterior,
      IN_highlow = input$IN_highlow, IN_perCap = input$IN_perCap
    )
  })

  # Update Expected Prior if Default Prior is Pressed.
  shiny::observeEvent(input$IN_currentDefault, {
    TEMP$ExpPrior <- updateExpectedPrior(
      IN_kiaThreshold = c(input$IN_kiaThreshold1, input$IN_kiaThreshold2),
      IN_datesPosterior = input$IN_datesPosterior,
      IN_highlow = input$IN_highlow, IN_perCap = input$IN_perCap
    )

    shiny::updateNumericInput(
      session,
      inputId = "IN_par1", value = switch(
        input$IN_distribution,
        "gamma" = TEMP$ExpPrior$Number,
        "normal" = defSignif(TEMP$ExpPrior$Mean),
        "uniform" = c(0, 7)
      ),
    )

    shiny::updateNumericInput(
      session,
      inputId = "IN_par2", value = switch(
        input$IN_distribution,
        "gamma" = TEMP$ExpPrior$Years,
        "normal" = defSignif(TEMP$ExpPrior$StdDev),
        "uniform" = 7
      )
    )
  })

  output$OUT_par1 <- shiny::renderUI({
    if (input$IN_distribution == "uniform") {

      shiny::sliderInput(
        "IN_par1",
        "Minimum and maximum arrival rates",
        min = 0, max = 7, value = c(0, 7), step = 0.01
      )

    } else {

      shiny::numericInput(
        "IN_par1",
        switch(input$IN_distribution,
               "gamma" = "Effective number of wars",
               "normal" = "Expected arrival rate",
               "uniform" = "Minimum arrival rate"
        ),
        min = switch(
          input$IN_distribution,
          "gamma" = 0,
          "normal" = 0,
          "uniform" = 0
        ),
        max = switch(
          input$IN_distribution,
          "gamma" = 1000,
          "normal" = 7,
          "uniform" = 7 # wars in one year
        ),
        value = switch(
          input$IN_distribution,
          "gamma" = TEMP$ExpPrior$Number,
          "normal" = TEMP$ExpPrior$Mean,
          "uniform" = 0
        )
      )
    }
  })

  output$OUT_par2 <- shiny::renderUI({
    if (input$IN_distribution == "uniform") {


    } else {
      shiny::numericInput(
        "IN_par2",
        switch(input$IN_distribution,
               "gamma" = "Effective number of years",
               "normal" = "Standard deviation of arrival rate",
               "uniform" = "Maximum arrival rate"
        ),
        min = switch(
          input$IN_distribution,
          "gamma" = 0,
          "normal" = 0,
          "uniform" = 0
        ),
        max = switch(
          input$IN_distribution,
          "gamma" = 1000,
          "normal" = 7,
          "uniform" = 7 # wars in one year
        ),
        value = switch(
          input$IN_distribution,
          "gamma" = TEMP$ExpPrior$Years,
          "normal" = TEMP$ExpPrior$StdDev,
          "uniform" = 7
        )
      )
    }
  })

  output$OUT_priorSummary <- shiny::renderUI({
    equivalence <- switch(
      input$IN_distribution,
      # mn = al / be, sd = sqrt(al/be^2)
      "gamma" = defFormat(c(input$IN_par1/ input$IN_par2,
                            sqrt(input$IN_par1/ (input$IN_par2^2)))),
      # al = (mn / sd)^2
      # be = mn / (sd^2)
      "normal" = defFormat(c((input$IN_par1/ input$IN_par2)^2,
                             input$IN_par1 / (input$IN_par2^2))),
      "uniform" = c("", "")
    )

    shiny::renderText({
      switch(
        input$IN_distribution,
        "gamma" = paste0(
          "This prior has a mean of ", equivalence[1],
          " and a standard deviation of ", equivalence[2], "."
        ),
        "normal" = paste(
          "A gamma with this mean and standard deviation would have",
          equivalence[1], "effective wars over",
          equivalence[2], "years."
        ),
        "uniform" = paste(equivalence, collapse = " "))
    })
  })

  ### Debugging Bits: ##########################################################
  output$OUT_Setup <- shiny::renderText(
    c(minkialow, minkialow_ph, maxkiahigh, maxkiahigh_ph)
  )
  output$OUT_Input <- shiny::renderText(toString(input$IN_perCap))

  ### Main: ####################################################################
  plotResults <- function(
    posteriorSamples, priorRateLocs, priorRateDens,
    ablineLoc, ablineLoc2, ablineLoc3
  ) {
    dens(posteriorSamples, xlim = range(priorRateLocs),
         show.HPDI = input$IN_ciWidth)
    lines(priorRateLocs, priorRateDens, col = "blue")
    abline(v = ablineLoc, lty = "dashed", col = "blue")
    abline(v = ablineLoc2, lty = "dotted", col = "red")
    abline(v = ablineLoc3, lty = "dotted", col = "black")
    legend(
      "topleft",
      c("Posterior", "Prior", "Prior Mean", "Historical Mean", "Post Data Mean",
        paste0(input$IN_ciWidth*100, "% Cred. Interval")),
      lty = c("solid", "solid", "dashed", "dotted", "dotted", NA),
      col = c("black", "blue", "blue", "red", "black", NA),
      fill = c(NA, NA, NA, NA, NA, "grey"),
      border = c(NA, NA, NA, NA, NA, "black")
    )
    # legend( #It might be possible to double up on legends to avoid the offset.
    #   "topleft",
    #   c("Posterior", "Prior", "Prior Mean", "Historical Mean", "Post Data Mean",
    #     paste0(input$IN_ciWidth*100, "% Cred. Interval")),
    #   fill = c(NA, NA, NA, NA, NA, "grey"),
    #   border = c(NA, NA, NA, NA, NA, "black")
    # )
  }

  shiny::observeEvent(input$IN_startComputation, {
    # print(input$IN_distribution)
    # print(input$IN_kiaThreshold1)
    # print(input$IN_kiaThreshold2)
    # print(input$IN_datesPosterior)
    # print(input$IN_highlow)
    # print(input$IN_perCap)

    set.seed(input$IN_seed)

    IN_par1 <- input$IN_par1[1]
    if(input$IN_distribution == "uniform") {
      IN_par2 <- input$IN_par1[2]
    } else {
      IN_par2 <- input$IN_par2
    }
    if (is.null(IN_par1)) {
      IN_par1 <- switch( # Ctrl C from observe event current default.
        input$IN_distribution,
        "gamma" = TEMP$ExpPrior$Number,
        "normal" = defSignif(TEMP$ExpPrior$Mean),
        "uniform" = c(0, 7)
      )
    }
    if (is.null(IN_par2)) {
      IN_par2 <- switch(
        input$IN_distribution,
        "gamma" = TEMP$ExpPrior$Years,
        "normal" = defSignif(TEMP$ExpPrior$StdDev),
        "uniform" = 7
      )
    }

    IN_kiaThreshold <- c(input$IN_kiaThreshold1, input$IN_kiaThreshold2)

    model <- switch(
      input$IN_distribution,
      "gamma" = alist(
        WarCounts ~ dpois(Rate),
        Rate ~ dgamma(shape = IN_par1, rate = IN_par2)
      ),
      "normal" = alist(
        WarCounts ~ dpois(Rate),
        Rate ~ dnorm(mean = IN_par1, sd = IN_par2)
      ),
      "uniform" = alist(
        WarCounts ~ dpois(Rate),
        Rate ~ dunif(min = IN_par1, max = IN_par2)
      )
    )

    dat <- warsizes

    if (input$IN_highlow == "Low" && input$IN_perCap == "Raw") {
      dat$Size <- dat$kialow
    }
    if (input$IN_highlow == "High" && input$IN_perCap == "Raw") {
      dat$Size <- dat$kiahigh
    }
    if (input$IN_highlow == "Low" && input$IN_perCap == "Per Capita") {
      dat$Size <- dat$kialow_ph
    }
    if (input$IN_highlow == "High" && input$IN_perCap == "Per Capita") {
      dat$Size <- dat$kiahigh_ph
    }

    # print(range(dat$Size))
    # print(range(dat$startdate))
    # print(str(dat))

    dat <- dat %>% dplyr::filter(
      Size >= IN_kiaThreshold[1],
      Size <= IN_kiaThreshold[2],
      startdate >= input$IN_datesPosterior[1],
      startdate <= input$IN_datesPosterior[2]
    ) %>% dplyr::group_by(
      startyear
    ) %>% dplyr::summarise(
      WarCounts = dplyr::n()
    ) %>% tidyr::complete(
      startyear = seq(as.numeric(strtrim(input$IN_datesPosterior[1], 4)),
                      as.numeric(strtrim(input$IN_datesPosterior[2], 4)),
                      1),
      fill = list(WarCounts = 0)
    )

    dat <- list(
      startyear = dat$startyear,
      WarCounts = dat$WarCounts,
      IN_par1 = IN_par1,
      IN_par2 = IN_par2
    )

    # print(range(dat$WarCounts))
    # print(range(dat$startyear))
    # print(str(dat))

    TEMP$Error <- tryCatch(
      {
        TEMP$Quap <- rethinking::quap(
          model, data = dat
        ); NULL
      },
      error = function(e) {
        print(e)
        return(e)
      },
      warning = function(w) {
        print(w)
        return(w)
      }
    )

    if (is.null(TEMP$Error)) {
      output$OUT_Error <- shiny::renderText("")

      ablineLoc <- switch(
        input$IN_distribution,
        "gamma" = IN_par1/IN_par2,
        "normal" = IN_par1,
        "uniform" = (IN_par1 + IN_par2)/2
      )

      ablineLoc2 <- TEMP$ExpPrior$Mean

      ablineLoc3 <- sum(dat$WarCounts)/length(dat$startyear)

      ablineLocs <- c(ablineLoc, ablineLoc2, ablineLoc3)

      posteriorSamples <- rethinking::extract.samples(TEMP$Quap)
      priorRateLocs <- seq(min(c(posteriorSamples$Rate, ablineLocs))*0.9,
                           max(c(posteriorSamples$Rate, ablineLocs))*1.1,
                           by = 0.001)
      priorRateDens <- switch(
        input$IN_distribution,
        "gamma" = dgamma(priorRateLocs, shape = IN_par1, rate = IN_par2),
        "normal" = dnorm(priorRateLocs, mean = IN_par1, sd = IN_par2),
        "uniform" = dunif(priorRateLocs, min = IN_par1, max = IN_par2)
      )

      toRender <- rethinking::precis(posteriorSamples, prob = input$IN_ciWidth)

      output$OUT_QuapPars1 <- shiny::renderText(
        c("Parameter-| ", names(toRender), "P(Posterior < Prior)")
      )
      output$OUT_QuapPars2 <- shiny::renderText(
        c(
          "Rate--------| ",
          formatC(unlist(toRender@.Data[1:4]), digits = 4, format = "g"),
          toRender@.Data[[5]],
          sum(posteriorSamples$Rate < rgamma(
            n = nrow(posteriorSamples),
            shape = IN_par1,
            rate = IN_par2)
          )/nrow(posteriorSamples)
        )
      )

      TEMP$Results <- list(
        posteriorSamples = posteriorSamples,
        priorRateLocs = priorRateLocs,
        priorRateDens = priorRateDens,
        ablineLoc = ablineLoc,
        ablineLoc2 = ablineLoc2,
        ablineLoc3 = ablineLoc3
      )

      TEMP$ResultsFlag <- TRUE

      output$OUT_QuapPlot <- shiny::renderPlot(plotResults(
        posteriorSamples, priorRateLocs, priorRateDens,
        ablineLoc, ablineLoc2, ablineLoc3))

    } else {
      # Error handling.
      output$OUT_Error <- shiny::renderPrint(TEMP$Error)
      output$OUT_QuapPars1 <- shiny::renderPrint("")
      output$OUT_QuapPars2 <- shiny::renderPrint("")
      # output$OUT_Advice <- switch()

    }
  })

  output$downloadImage <- shiny::downloadHandler(
    filename = function() {
      paste0("AppPlot ",
             gsub(pattern = ":", replacement = "-", Sys.time(), fixed = TRUE),
             ".png")
    },
    content = function(file) {
      if(!is.na(TEMP$ResultsFlag)) {
        png(filename = file)
        with(TEMP$Results,
             plotResults(
               posteriorSamples, priorRateLocs, priorRateDens,
               ablineLoc, ablineLoc2, ablineLoc3
             )
        )
        dev.off()
      }
    },
    contentType = "image/png"
  )
}

# Run the application: #########################################################
shinyApp(ui = ui, server = server)
