#Load the packages

library(tidyverse)

#Here is the documentation for the package used to test the hypothesis that
#war arrivals fit a Poisson distribution and here is a link to the package
#https://www.rdocumentation.org/packages/iZID/versions/0.0.1/topics/dis.kstest

library(iZID)

#Load the Mars war size data

warsizes <- read.csv("warsizes.csv")


#There are a two small calculations of p values in the first paragraph of 
#section 5. Note that these are just two among many calculations that go 
#into figure 5 so the code is already there but perhaps it's helpful to show 
#it here as well.

#The probability of 49 wars or fewer during 61 years with an annual arrival rate
#of 1.34 war per year (equal to the pre-1950 war arrival rate) and assuming a 
#that war arrivals follow a poisson distribution is less than 0.00001:

ppois(49, 1.34*61)

#The probability of 28 wars or fewer during 61 years with an annual arrival rate
#of 0.65 war per year (equal to the pre-1950 war arrival rate) and assuming a 
#that war arrivals follow a poisson distribution is a about 0.033:

ppois(28, 0.65*61)

#Early in section 5 there is a footnote reporting on tests of the hypothesis that
#war arrivals, over the whole period and for the pre-and-post-1950 sub-periods
#follow Poisson distributions. Here is the code.

starts <- function(i){
  nrow(filter(warsizes,startyear == i))
}  

start_hist_all <- map_dbl(1800:2011,starts)
start_hist_pre <- map_dbl(1800:1950, starts)
start_hist_post <- map_dbl(1951:2011, starts)

set.seed(1)

dis.kstest(start_hist_all,nsim=10000,bootstrap=TRUE,distri='Poisson')$pvalue

dis.kstest(start_hist_pre,nsim=10000,bootstrap=TRUE,distri='Poisson')$pvalue

dis.kstest(start_hist_post,nsim=10000,bootstrap=TRUE,distri='Poisson')$pvalue

#The Poisson is not rejected - It seems OK to proceed using the Poisson.

#Later in section 5 there is a footnote in section 5 in which we report 
#on reverse calculations for which our ranges are from a maximum point 
#down to 0 rather than from a minimum point up to infinity as in the 
#rest of the paper. Here is the code supporting that footnote.

#Create a vector giving the number of wars below the size of each post-1950 war
number_under_low <- c()
for(i in 1:252) {
  if(warsizes[i,"startyear"] > 1950) {
    temporary <- warsizes[i, "kialow"]
    number_under_low <- c(number_under_low, 
                          nrow(filter(warsizes, startyear < 1951, kialow < temporary)) )
  }
}


#The idea here is that for each post-1950 war you calculate the number of 
#post-1950 wars there should be at or below that size if post-1950 wars in 
#that size range arrive at the rate they did before 1950

number_under_low_post <- c(1:length(number_under_low))
number_under_low_sorted <- sort(number_under_low)
pred_under_low_central <- (number_under_low_sorted)*61/151
pred_under_low_upper  <- qpois(.975, pred_under_low_central, lower.tail = TRUE)
pred_under_low_lower  <- qpois(.025, pred_under_low_central, lower.tail = TRUE)
pred_under_low_lower_.05 <- qpois(.05, pred_under_low_central, lower.tail = TRUE)
pred_under_low_lower_.10 <- qpois(.10, pred_under_low_central, lower.tail = TRUE)

pred_under_low_lower - c(1:49)

#This shows that when we reach the size of the 20th biggest post-1950 war 
#then we can reject the no-change hypothesis at the 2.5% level i.e, 
#reject the idea that wars of this size or smaller are generated at the 
#same rate as they werebefore 1950.  This happens at KIALOW of 2,700

sort(filter(warsizes, startyear >1950)$kialow)

#Now repeat the procedures for KIAHIGH

#Create a vector giving the number of wars below the size of each post-1950 war
number_under_high <- c()
for(i in 1:252) {
  if(warsizes[i,"startyear"] > 1950) {
    temporary <- warsizes[i, "kiahigh"]
    number_under_high <- c(number_under_high, 
                           nrow(filter(warsizes, startyear < 1951, kiahigh < temporary)) )
  }
}



#The idea here is that for each post-1950 war you calculate the number of 
#post-1950 wars there should be at or below that size if post-1950 wars in 
#that size range arrive at the rate they did before 1950

number_under_high_post <- c(1:length(number_under_high))
number_under_high_sorted <- sort(number_under_high)
pred_under_high_central <- (number_under_high_sorted)*61/151
pred_under_high_upper  <- qpois(.975, pred_under_high_central, lower.tail = TRUE)
pred_under_high_lower  <- qpois(.025, pred_under_high_central, lower.tail = TRUE)
pred_under_high_lower_.05 <- qpois(.05, pred_under_high_central, lower.tail = TRUE)
pred_under_high_lower_.10 <- qpois(.10, pred_under_high_central, lower.tail = TRUE)


pred_under_high_lower - c(1:49)

#This shows that when we reach the size of the 18thth biggest post-1950 war then we
#can reject the no-change hypothesis at the 2.5% level i.e, reject the idea that
#that wars of this size or smaller are generated at the same rate as they were
#before 1950.  This happens at KIAHIGH of 5,500

sort(filter(warsizes, startyear >1950)$kiahigh)


#The rest of this file covers the computations presented in Appendix 2

#Load the packages

library(tidyverse)

library(poweRlaw)
library(gamlss)



#The following bit of code is to derive 1 number that appears in section 4 of the
#paper, specifically, that the Kolmogorov-Smirnov statistic on the power law
#fit to the CoW warsize data is 0.63, suggesting that it is at least plausible
#that the data was generated by a power law although, as we shall see, other
#evidence is against the power law hypothesis.

#Load the CoW data

warsizes_CoW <- read.csv("warsizes_CoW.csv")

#Get set up for power law estimate

all_CoW <- warsizes_CoW$deaths
all_CoW_pl   <- displ$new(all_CoW)

#We now fit a power law to the data. Note - takes about 20 minutes and was already
#done in the data prep routine

(est_all_CoW_pl <- estimate_xmin(all_CoW_pl, xmax = 100000000))

#And the weird step flagged in the Data Prep routine where you have to tell the
#power law object what xmin is

all_CoW_pl$setXmin(est_all_CoW_pl)

#Now use the bootstrap command to estimate the KS statistic on the power law fit
#Unfortunately, this takes about 3 hours on my laptop
#Note - there should be an easier way to get the KS statistics but this is all
#I could find

bs_pl_all_CoW_test <- 
  bootstrap_p(all_CoW_pl, xmins = seq(1000,10000,100),xmax = 100000000,
              no_of_sims = 100, threads = 4, seed = 1)

#The p comes out at 0.63.  It was only 100 bootstraps with xmin constrained to 
#be between 1,000 and 10,000 so we shouldn't place too much weight on this but 
#it is sufficient for our purposes.

#Save the results

write.csv(bs_pl_all_CoW_test$bootstraps, "CoW_bootstrap.csv")

#Now let's do Vuong's test to compare the power law with a lognormal. We have to
#set xmin in the lognormal equal to the estimated xmin for the power law 
#(which is 4,985)

all_CoW_ln <- dislnorm$new(all_CoW)
all_CoW_ln$setXmin(all_CoW_pl$getXmin())
est <- estimate_pars(all_CoW_ln)
all_CoW_ln$setPars(est)
comp_pl_ln_CoW <- compare_distributions(all_CoW_pl, all_CoW_ln)

#The test statistics is -0.33 so really a dead heat between the power law and
#the lognormal

#Now let's compare to with the exponential

all_CoW_exp <- disexp$new(all_CoW)
all_CoW_exp$setXmin(all_CoW_pl$getXmin())
est <- estimate_pars(all_CoW_exp)
all_CoW_exp$setPars(est)
comp_pl_exp_CoW <- compare_distributions(all_CoW_pl, all_CoW_exp)

#Next we use the Gamlss package to fit a bunch of fat-tailed distributions and
#rank them according to their AIC statistics

warsizes_CoW_deaths <- sort(warsizes_CoW$deaths, decreasing = FALSE)

m1 <- fitDist(warsizes_CoW_deaths)
m1$fits
m1$fails

#The best fit is a generalized gamma.  Pareto2, a 2-parameter power law, ranks 11th
#The lognormal ranks 13th.  The best fit has an AIC of 2140.129 compared to 2174.223
#for the Pareto2 and 2185.690 for the lognormal. 

#Now we repeat the above comparisons of distributions for the low and high 
#estimates of KIA in the Mars data

#First we get set up but, again, note that if you've just run the data prep
#routine you don't have to run this again (It takes about 20 minutes)

#Get set up for power law estimate

all_low <- warsizes$kialow
all_low_pl   <- displ$new(all_low)

#We now fit a power law to the data. Note - takes about 20 minutes and was already
#done in the data prep routine

(est_all_low_pl <- estimate_xmin(all_low_pl, xmax = 100000000))
all_low_pl$setXmin(est_all_low_pl)

#First we do Vuong's test to compare the power law with a lognormal for
#which we set xmin in the lognormal equal to the estimated xmin for the
#power law (which is 6,000)

all_low_ln <- dislnorm$new(all_low)
all_low_ln$setXmin(all_low_pl$getXmin())
est <- estimate_pars(all_low_ln)
all_low_ln$setPars(est)
comp_pl_ln_low <- compare_distributions(all_low_pl, all_low_ln)

#Now let's compare to with the exponential

all_low_exp <- disexp$new(all_low)
all_low_exp$setXmin(all_low_pl$getXmin())
est <- estimate_pars(all_low_exp)
all_low_exp$setPars(est)
comp_pl_exp_low <- compare_distributions(all_low_pl, all_low_exp)

#And now we do the high estimates starting with the set up

all_high <- warsizes$kiahigh
all_high_pl   <- displ$new(all_high)

#We now fit a power law to the data. Note - takes about 20 minutes and was already
#done in the data prep routine

(est_all_high_pl <- estimate_xmin(all_high_pl, xmax = 100000000))
all_high_pl$setXmin(est_all_high_pl)

#First we do Vuong's test to compare the power law with a lognormal for
#which we set xmin in the lognormal equal to the estimated xmin for the
#power law (which is 10,575)

all_high_ln <- dislnorm$new(all_high)
all_high_ln$setXmin(all_high_pl$getXmin())
est <- estimate_pars(all_high_ln)
all_high_ln$setPars(est)
comp_pl_ln_high <- compare_distributions(all_high_pl, all_high_ln)

#Now let's compare to with the exponential

all_high_exp <- disexp$new(all_high)
all_high_exp$setXmin(all_high_pl$getXmin())
est <- estimate_pars(all_high_exp)
all_high_exp$setPars(est)
comp_pl_exp_high <- compare_distributions(all_high_pl, all_high_exp)

#It's worth noting that the results of this comparison are extremely similar to
#but not identical to the results for the low estimates of KIA

#Now we move on to the Gamlss analysis for both the low and high estimates of
#Mars KIA

mars_war_sizes_low <- sort(warsizes$kialow, decreasing = FALSE)
mars_war_sizes_high <- sort(warsizes$kiahigh, decreasing = FALSE)

#war_sizes_high <- war_sizes_full[11:95]

m2 <- fitDist(mars_war_sizes_low)
m2$fits
m2$fails

m3 <- fitDist(mars_war_sizes_high)
m3$fits
m3$fails

#The best fits are a generalized beta for and Box Cox t for both
#KIALOW and KIAHIGH but with the orders interchanged.
#Pareto2, a 2-parameter power law, ranks 6th for KIALOW and 5th for KIAHIGH
#The lognormal ranks and 10th for both KIALOW and KIAHIGH.  
#The best fit for KIALOW has an AIC of 5322.443 compared to 5326.598
#for the Pareto2 and 5346.789  for the lognormal. 
#The best fit for KIAHIGH has an AIC of 5570.511  compared to 5578.186 
#for the Pareto2 and 5600.074 for the lognormal.


#Now we move on to bunch of simulations which are presented in Table 1A of
#Appendix 2. THESE ARE VERY COMPUTATION INTENSIVE 
#SO DON'T DO THEM UNLESS YOU HAVE A LOT OF PATIENCE AND COMPUATION POWER. THEY
#TOOK US ABOUT 10 DAYS ON THE GOOGLE CLOUD

#We use the Gillespie PL package to generate samples of 50 draws from power
#law distributions, always with xmin = 1,000, but with exponents equal to
# 1.3, 1.4, ..., 2.3, 2.4.  Over the course of all the simulations there are
#200 50-draw samples for each of the 12 exponent values.

#We did it in 4 chunks so that if something went wrong we could try again before
#waiting too long. So the first and second chuncks of code are identical except
#for the seed.  The same goes for the third and fourth chunks.

#This is the first chunk of code. For each alpha in 
# c(1.5, 1.6, 1.7, 1.8, 1.9, 2.0) there are 100 sets of 50 observations. 
#xmin was set at 1,000 
#discrete_max is set to 20,000,000.  This just means that the algorithm generates
#genuine discrete power law draws up to 20,000,000 and then resorts to treating
#the power law like a continuous one beyond that. In other words, the random numbers
#below 20 million are higher in quality than those above 20 million. This seems plenty
#high since the largest war in CoW has about 17 million battle deaths.

library(poweRlaw)

set.seed(199)

my_data <- c()

for (j in c(1.5, 1.6, 1.7, 1.8, 1.9, 2.0)) {
  for (i in 1:100) {
    my_data <- c(my_data,rpldis(50, 1000, j, discrete_max = 20000000))
  }
}

pl_data <- as.data.frame(matrix(my_data, nrow = 5000, ncol = 6))
colnames(pl_data) <- c("1.5", "1.6", "1.7", "1.8", "1.9", "2.0")


slopes <- c()
minima <- c()
for (k in 1:600) {
  pl_object <- displ$new(my_data[1:50+50*(k-1)])
  pl_fit <- estimate_xmin(pl_object, xmax = 20000000)
  slopes <- c(slopes, pl_fit$pars)
  minima <- c(minima, pl_fit$xmin)
}

alphas <- as.data.frame(matrix(slopes, nrow =100, ncol = 6))
colnames(alphas) <- c("1.5", "1.6", "1.7", "1.8", "1.9", "2.0")

x_mins <- as.data.frame(matrix(minima, nrow = 100, ncol = 6))
colnames(x_mins) <- c("1.5", "1.6", "1.7", "1.8", "1.9", "2.0")

write.csv(alphas, "alphas1.csv")
write.csv(x_mins, "x_mins1.csv")
write.csv(pl_data, "pl_data1.csv")

#Here are the second set of simulations which are identical to the first except
#for the seed

library(poweRlaw)

set.seed(299)

my_data <- c()

for (j in c(1.5, 1.6, 1.7, 1.8, 1.9, 2.0)) {
  for (i in 1:100) {
    my_data <- c(my_data,rpldis(50, 1000, j, discrete_max = 20000000))
  }
}

pl_data <- as.data.frame(matrix(my_data, nrow = 5000, ncol = 6))
colnames(pl_data) <- c("1.5", "1.6", "1.7", "1.8", "1.9", "2.0")


slopes <- c()
minima <- c()
for (k in 1:600) {
  pl_object <- displ$new(my_data[1:50+50*(k-1)])
  pl_fit <- estimate_xmin(pl_object, xmax = 20000000)
  slopes <- c(slopes, pl_fit$pars)
  minima <- c(minima, pl_fit$xmin)
}

alphas <- as.data.frame(matrix(slopes, nrow =100, ncol = 6))
colnames(alphas) <- c("1.5", "1.6", "1.7", "1.8", "1.9", "2.0")

x_mins <- as.data.frame(matrix(minima, nrow = 100, ncol = 6))
colnames(x_mins) <- c("1.5", "1.6", "1.7", "1.8", "1.9", "2.0")

write.csv(alphas, "alphas2.csv")
write.csv(x_mins, "x_mins2.csv")
write.csv(pl_data, "pl_data2.csv")

#Here is the third set of simulations followed immediately by the fourths. These
#repeat what we just did but for different values of the power law exponent

library(poweRlaw)

set.seed(399)

my_data <- c()

for (j in c(1.3, 1.4, 2.1, 2.2, 2.3, 2.4)) {
  for (i in 1:100) {
    my_data <- c(my_data,rpldis(50, 1000, j, discrete_max = 20000000))
  }
}

pl_data <- as.data.frame(matrix(my_data, nrow = 5000, ncol = 6))
colnames(pl_data) <- c("1.3", "1.4", "2.1", "2.2", "2.3", "2.4")


slopes <- c()
minima <- c()
for (k in 1:600) {
  pl_object <- displ$new(my_data[1:50+50*(k-1)])
  pl_fit <- estimate_xmin(pl_object, xmax = 20000000)
  slopes <- c(slopes, pl_fit$pars)
  minima <- c(minima, pl_fit$xmin)
}

alphas <- as.data.frame(matrix(slopes, nrow =100, ncol = 6))
colnames(alphas) <- c("1.3", "1.4", "2.1", "2.2", "2.3", "2.4")

x_mins <- as.data.frame(matrix(minima, nrow = 100, ncol = 6))
colnames(x_mins) <- c("1.3", "1.4", "2.1", "2.2", "2.3", "2.4")

write.csv(alphas, "alphas3.csv")
write.csv(x_mins, "x_mins3.csv")
write.csv(pl_data, "pl_data3.csv")

library(poweRlaw)

set.seed(499)

my_data <- c()

for (j in c(1.3, 1.4, 2.1, 2.2, 2.3, 2.4)) {
  for (i in 1:100) {
    my_data <- c(my_data,rpldis(50, 1000, j, discrete_max = 20000000))
  }
}

pl_data <- as.data.frame(matrix(my_data, nrow = 5000, ncol = 6))
colnames(pl_data) <- c("1.3", "1.4", "2.1", "2.2", "2.3", "2.4")


slopes <- c()
minima <- c()
for (k in 1:600) {
  pl_object <- displ$new(my_data[1:50+50*(k-1)])
  pl_fit <- estimate_xmin(pl_object, xmax = 20000000)
  slopes <- c(slopes, pl_fit$pars)
  minima <- c(minima, pl_fit$xmin)
}

alphas <- as.data.frame(matrix(slopes, nrow =100, ncol = 6))
colnames(alphas) <- c("1.3", "1.4", "2.1", "2.2", "2.3", "2.4")

x_mins <- as.data.frame(matrix(minima, nrow = 100, ncol = 6))
colnames(x_mins) <- c("1.3", "1.4", "2.1", "2.2", "2.3", "2.4")

write.csv(alphas, "alphas4.csv")
write.csv(x_mins, "x_mins4.csv")
write.csv(pl_data, "pl_data4.csv")

#The data collected in these simulations is merged into a single dataset in 
#the Data Prep routine and is analyzed and summarized as Table 1A in 
#Appendix 2





